@@ -1,5 +1,5 @@
-// Copyright (C) 2004, 2007 Nathaniel Smith <njs@pobox.com>
-// Copyright (C) 2007 - 2008 Stephen Leake <stephen_leake@stephe-leake.org>
+// Copyright (C) 2004 Nathaniel Smith <njs@pobox.com>
+//               2007 Stephen Leake <stephen_leake@stephe-leake.org>
 //
 // This program is made available under the GNU GPL version 2.0 or
 // greater. See the accompanying file COPYING for details.
@@ -17,7 +17,7 @@
 
 #include <boost/bind.hpp>
 #include <boost/function.hpp>
-#include <boost/lexical_cast.hpp>
+#include "lexical_cast.hh"
 #include <boost/tuple/tuple.hpp>
 
 #include "app_state.hh"
@@ -44,6 +44,7 @@
 #include "work.hh"
 #include "xdelta.hh"
 #include "database.hh"
+#include "vocab_cast.hh"
 
 using std::allocator;
 using std::basic_ios;
@@ -77,7 +78,7 @@ CMD_AUTOMATE(heads, N_("[BRANCH]"),
              "",
              options::opts::none)
 {
-  N(args.size() < 2,
+  E(args.size() < 2, origin::user,
     F("wrong argument count"));
 
   database db(app);
@@ -86,11 +87,11 @@ CMD_AUTOMATE(heads, N_("[BRANCH]"),
   branch_name branch;
   if (args.size() == 1)
     // branchname was explicitly given, use that
-    branch = branch_name(idx(args, 0)());
+    branch = typecast_vocab<branch_name>(idx(args, 0));
   else
     {
       workspace::require_workspace(F("with no argument, this command prints the heads of the workspace's branch"));
-      branch = app.opts.branchname;
+      branch = app.opts.branch;
     }
 
   set<revision_id> heads;
@@ -114,7 +115,7 @@ CMD_AUTOMATE(ancestors, N_("REV1 [REV2 [REV3 [...]]]"),
              "",
              options::opts::none)
 {
-  N(args.size() > 0,
+  E(args.size() > 0, origin::user,
     F("wrong argument count"));
 
   database db(app);
@@ -123,8 +124,9 @@ CMD_AUTOMATE(ancestors, N_("REV1 [REV2 [REV3 [...]]]"),
   vector<revision_id> frontier;
   for (args_vector::const_iterator i = args.begin(); i != args.end(); ++i)
     {
-      revision_id rid(decode_hexenc((*i)()));
-      N(db.revision_exists(rid), F("no such revision '%s'") % rid);
+      revision_id rid(decode_hexenc_as<revision_id>((*i)(), origin::user));
+      E(db.revision_exists(rid), origin::user,
+        F("no such revision '%s'") % rid);
       frontier.push_back(rid);
     }
   while (!frontier.empty())
@@ -166,7 +168,7 @@ CMD_AUTOMATE(descendents, N_("REV1 [REV2 [REV3 [...]]]"),
              "",
              options::opts::none)
 {
-  N(args.size() > 0,
+  E(args.size() > 0, origin::user,
     F("wrong argument count"));
 
   database db(app);
@@ -175,8 +177,9 @@ CMD_AUTOMATE(descendents, N_("REV1 [REV2 [REV3 [...]]]"),
   vector<revision_id> frontier;
   for (args_vector::const_iterator i = args.begin(); i != args.end(); ++i)
     {
-      revision_id rid(decode_hexenc((*i)()));
-      N(db.revision_exists(rid), F("no such revision '%s'") % rid);
+      revision_id rid(decode_hexenc_as<revision_id>((*i)(), origin::user));
+      E(db.revision_exists(rid), origin::user,
+        F("no such revision '%s'") % rid);
       frontier.push_back(rid);
     }
   while (!frontier.empty())
@@ -224,8 +227,9 @@ CMD_AUTOMATE(erase_ancestors, N_("[REV1 [REV2 [REV3 [...]]]]"),
   set<revision_id> revs;
   for (args_vector::const_iterator i = args.begin(); i != args.end(); ++i)
     {
-      revision_id rid(decode_hexenc((*i)()));
-      N(db.revision_exists(rid), F("no such revision '%s'") % rid);
+      revision_id rid(decode_hexenc_as<revision_id>((*i)(), origin::user));
+      E(db.revision_exists(rid), origin::user,
+        F("no such revision '%s'") % rid);
       revs.insert(rid);
     }
   erase_ancestors(db, revs);
@@ -253,8 +257,9 @@ CMD_AUTOMATE(toposort, N_("[REV1 [REV2 [REV3 [...]]]]"),
   set<revision_id> revs;
   for (args_vector::const_iterator i = args.begin(); i != args.end(); ++i)
     {
-      revision_id rid(decode_hexenc((*i)()));
-      N(db.revision_exists(rid), F("no such revision '%s'") % rid);
+      revision_id rid(decode_hexenc_as<revision_id>((*i)(), origin::user));
+      E(db.revision_exists(rid), origin::user,
+        F("no such revision '%s'") % rid);
       revs.insert(rid);
     }
   vector<revision_id> sorted;
@@ -286,7 +291,7 @@ CMD_AUTOMATE(ancestry_difference, N_("NEW_REV [OLD_REV1 [OLD_REV2 [...]]]"),
              "",
              options::opts::none)
 {
-  N(args.size() > 0,
+  E(args.size() > 0, origin::user,
     F("wrong argument count"));
 
   database db(app);
@@ -294,12 +299,14 @@ CMD_AUTOMATE(ancestry_difference, N_("NEW_REV [OLD_REV1 [OLD_REV2 [...]]]"),
   revision_id a;
   set<revision_id> bs;
   args_vector::const_iterator i = args.begin();
-  a = revision_id(decode_hexenc((*i)()));
-  N(db.revision_exists(a), F("no such revision '%s'") % a);
+  a = decode_hexenc_as<revision_id>((*i)(), origin::user);
+  E(db.revision_exists(a), origin::user,
+    F("no such revision '%s'") % a);
   for (++i; i != args.end(); ++i)
     {
-      revision_id b(decode_hexenc((*i)()));
-      N(db.revision_exists(b), F("no such revision '%s'") % b);
+      revision_id b(decode_hexenc_as<revision_id>((*i)(), origin::user));
+      E(db.revision_exists(b), origin::user,
+        F("no such revision '%s'") % b);
       bs.insert(b);
     }
   set<revision_id> ancestors;
@@ -330,7 +337,7 @@ CMD_AUTOMATE(leaves, "",
              "",
              options::opts::none)
 {
-  N(args.size() == 0,
+  E(args.size() == 0, origin::user,
     F("no arguments needed"));
 
   database db(app);
@@ -356,7 +363,7 @@ CMD_AUTOMATE(roots, "",
              "",
              options::opts::none)
 {
-  N(args.size() == 0,
+  E(args.size() == 0, origin::user,
     F("no arguments needed"));
 
   database db(app);
@@ -386,13 +393,14 @@ CMD_AUTOMATE(parents, N_("REV"),
              "",
              options::opts::none)
 {
-  N(args.size() == 1,
+  E(args.size() == 1, origin::user,
     F("wrong argument count"));
 
   database db(app);
 
-  revision_id rid(decode_hexenc(idx(args, 0)()));
-  N(db.revision_exists(rid), F("no such revision '%s'") % rid);
+  revision_id rid(decode_hexenc_as<revision_id>(idx(args, 0)(), origin::user));
+  E(db.revision_exists(rid), origin::user,
+    F("no such revision '%s'") % rid);
   set<revision_id> parents;
   db.get_revision_parents(rid, parents);
   for (set<revision_id>::const_iterator i = parents.begin();
@@ -416,13 +424,14 @@ CMD_AUTOMATE(children, N_("REV"),
              "",
              options::opts::none)
 {
-  N(args.size() == 1,
+  E(args.size() == 1, origin::user,
     F("wrong argument count"));
 
   database db(app);
 
-  revision_id rid(decode_hexenc(idx(args, 0)()));
-  N(db.revision_exists(rid), F("no such revision '%s'") % rid);
+  revision_id rid(decode_hexenc_as<revision_id>(idx(args, 0)(), origin::user));
+  E(db.revision_exists(rid), origin::user,
+    F("no such revision '%s'") % rid);
   set<revision_id> children;
   db.get_revision_children(rid, children);
   for (set<revision_id>::const_iterator i = children.begin();
@@ -456,7 +465,7 @@ CMD_AUTOMATE(graph, "",
              "",
              options::opts::none)
 {
-  N(args.size() == 0,
+  E(args.size() == 0, origin::user,
     F("no arguments needed"));
 
   database db(app);
@@ -504,7 +513,7 @@ CMD_AUTOMATE(select, N_("SELECTOR"),
              "",
              options::opts::none)
 {
-  N(args.size() == 1,
+  E(args.size() == 1, origin::user,
     F("wrong argument count"));
 
   database db(app);
@@ -526,9 +535,9 @@ struct node_info
   node_id id;
   path::status type;
   file_id ident;
-  full_attr_map_t attrs;
+  attr_map_t attrs;
 
-  node_info() : exists(false), type(path::nonexistent) {}
+  node_info() : exists(false), id(the_null_node), type(path::nonexistent) {}
 };
 
 static void
@@ -830,6 +839,7 @@ namespace
     symbol const old_path("old_path");
     symbol const new_path("new_path");
     symbol const status("status");
+    symbol const birth("birth");
     symbol const changes("changes");
   }
 }
@@ -879,7 +889,17 @@ inventory_determine_states(workspace & work, file_path const & fs_path,
   if (item.fs_type == path::nonexistent)
     {
       if (item.new_node.exists)
-        states.push_back("missing");
+        {
+          states.push_back("missing");
+
+          // If this node is in a directory that is ignored in .mtn-ignore,
+          // we will output this warning. Note that we don't detect a known
+          // file that is ignored but not in an ignored directory.
+          if (work.ignore_file(fs_path))
+            W(F("'%s' is both known and ignored; "
+              "it will be shown as 'missing'. Check .mtn-ignore.")
+            % fs_path);
+        }
     }
   else // exists on filesystem
     {
@@ -955,11 +975,28 @@ inventory_determine_changes(inventory_item const & item, roster_t const & old_ro
       if (item.new_node.type == path::file)
         changes.push_back("content");
 
-      if (item.new_node.attrs.size() > 0)
+      if (!item.new_node.attrs.empty())
         changes.push_back("attrs");
     }
 }
 
+static revision_id
+inventory_determine_birth(inventory_item const & item,
+                          roster_t const & old_roster,
+                          marking_map const & old_marking)
+{
+  revision_id rid;
+  if (old_roster.has_node(item.new_node.id))
+    {
+      node_t node = old_roster.get_node(item.new_node.id);
+      marking_map::const_iterator m = old_marking.find(node->self);
+      I(m != old_marking.end());
+      marking_t mark = m->second;
+      rid = mark.birth_revision;
+    }
+  return rid;
+}
+
 // Name: inventory
 // Arguments: [PATH]...
 // Added in: 1.0
@@ -990,11 +1027,12 @@ CMD_AUTOMATE(inventory,  N_("[PATH]..."),
   work.get_parent_rosters(db, parents);
   // for now, until we've figured out what the format could look like
   // and what conceptional model we can implement
-  // see: http://www.venge.net/mtn-wiki/MultiParentWorkspaceFallout
-  N(parents.size() == 1,
+  // see: http://monotone.ca/wiki/MultiParentWorkspaceFallout/
+  E(parents.size() == 1, origin::user,
     F("this command can only be used in a single-parent workspace"));
 
   roster_t new_roster, old_roster = parent_roster(parents.begin());
+  marking_map old_marking = parent_marking(parents.begin());
   temp_node_id_source nis;
 
   work.get_current_roster_shape(db, nis, new_roster);
@@ -1017,11 +1055,13 @@ CMD_AUTOMATE(inventory,  N_("[PATH]..."),
            inserter(excludes, excludes.end()));
     }
 
-  node_restriction nmask(work, includes, excludes, app.opts.depth, old_roster, new_roster);
+  node_restriction nmask(includes, excludes, app.opts.depth,
+                         old_roster, new_roster, ignored_file(work));
   // skip the check of the workspace paths because some of them might
   // be missing and the user might want to query the recorded structure
   // of them anyways
-  path_restriction pmask(work, includes, excludes, app.opts.depth, path_restriction::skip_check);
+  path_restriction pmask(includes, excludes, app.opts.depth,
+                         path_restriction::skip_check);
 
   inventory_rosters(old_roster, new_roster, nmask, pmask, inventory);
   inventory_filesystem(work, pmask, inventory);
@@ -1052,6 +1092,9 @@ CMD_AUTOMATE(inventory,  N_("[PATH]..."),
       vector<string> changes;
       inventory_determine_changes(item, old_roster, changes);
 
+      revision_id birth_revision =
+        inventory_determine_birth(item, old_roster, old_marking);
+
       bool is_tracked =
         find(states.begin(), states.end(), "unknown") == states.end() &&
         find(states.begin(), states.end(), "ignored") == states.end();
@@ -1113,6 +1156,9 @@ CMD_AUTOMATE(inventory,  N_("[PATH]..."),
       //
       // finally output the previously recorded states and changes
       //
+      if (!birth_revision.inner()().empty())
+        st.push_binary_pair(syms::birth, birth_revision.inner());
+
       I(!states.empty());
       st.push_str_multi(syms::status, states);
 
@@ -1193,14 +1239,14 @@ CMD_AUTOMATE(get_revision, N_("REVID"),
              "",
              options::opts::none)
 {
-  N(args.size() == 1,
+  E(args.size() == 1, origin::user,
     F("wrong argument count"));
 
   database db(app);
 
   revision_data dat;
-  revision_id rid(decode_hexenc(idx(args, 0)()));
-  N(db.revision_exists(rid),
+  revision_id rid(decode_hexenc_as<revision_id>(idx(args, 0)(), origin::user));
+  E(db.revision_exists(rid), origin::user,
     F("no revision %s found in database") % rid);
   db.get_revision(rid, dat);
 
@@ -1214,8 +1260,7 @@ CMD_AUTOMATE(get_revision, N_("REVID"),
 // Added in: 7.0
 // Purpose: Outputs (an optionally restricted) revision based on
 //          changes in the current workspace
-// Error conditions: If there are no changes in the current workspace or the
-// restriction is invalid or has no recorded changes, prints an error message
+// Error conditions: If the restriction is invalid, prints an error message
 // to stderr and exits with status 1. A workspace is required.
 CMD_AUTOMATE(get_current_revision, N_("[PATHS ...]"),
              N_("Shows change information for a workspace"),
@@ -1247,8 +1292,7 @@ CMD_AUTOMATE(get_current_revision, N_("[PATHS ...]"),
   make_restricted_revision(old_rosters, new_roster, mask, rev,
                            excluded, join_words(execid));
   rev.check_sane();
-  N(rev.is_nontrivial(), F("no changes to commit"));
-  
+
   calculate_ident(rev, ident);
   write_revision(rev, dat);
 
@@ -1269,7 +1313,7 @@ CMD_AUTOMATE(get_base_revision_id, "",
              "",
              options::opts::none)
 {
-  N(args.size() == 0,
+  E(args.size() == 0, origin::user,
     F("no arguments needed"));
 
   database db(app);
@@ -1277,7 +1321,7 @@ CMD_AUTOMATE(get_base_revision_id, "",
 
   parent_map parents;
   work.get_parent_rosters(db, parents);
-  N(parents.size() == 1,
+  E(parents.size() == 1, origin::user,
     F("this command can only be used in a single-parent workspace"));
 
   output << parent_id(parents.begin()) << '\n';
@@ -1297,7 +1341,7 @@ CMD_AUTOMATE(get_current_revision_id, "",
              "",
              options::opts::none)
 {
-  N(args.size() == 0,
+  E(args.size() == 0, origin::user,
     F("no arguments needed"));
 
   workspace work(app);
@@ -1368,7 +1412,7 @@ CMD_AUTOMATE(get_manifest_of, N_("[REVID]"),
 {
   database db(app);
 
-  N(args.size() < 2,
+  E(args.size() < 2, origin::user,
     F("wrong argument count"));
 
   manifest_data dat;
@@ -1386,8 +1430,8 @@ CMD_AUTOMATE(get_manifest_of, N_("[REVID]"),
     }
   else
     {
-      revision_id rid = revision_id(decode_hexenc(idx(args, 0)()));
-      N(db.revision_exists(rid),
+      revision_id rid = decode_hexenc_as<revision_id>(idx(args, 0)(), origin::user);
+      E(db.revision_exists(rid), origin::user,
         F("no revision %s found in database") % rid);
       db.get_roster(rid, new_roster);
     }
@@ -1415,17 +1459,18 @@ CMD_AUTOMATE(packet_for_rdata, N_("REVID"),
              "",
              options::opts::none)
 {
-  N(args.size() == 1,
+  E(args.size() == 1, origin::user,
     F("wrong argument count"));
 
   database db(app);
 
   packet_writer pw(output);
 
-  revision_id r_id(decode_hexenc(idx(args, 0)()));
+  revision_id r_id(decode_hexenc_as<revision_id>(idx(args, 0)(), origin::user));
   revision_data r_data;
 
-  N(db.revision_exists(r_id), F("no such revision '%s'") % r_id);
+  E(db.revision_exists(r_id), origin::user,
+    F("no such revision '%s'") % r_id);
   db.get_revision(r_id, r_data);
   pw.consume_revision_data(r_id, r_data);
 }
@@ -1446,20 +1491,21 @@ CMD_AUTOMATE(packets_for_certs, N_("REVID"),
              "",
              options::opts::none)
 {
-  N(args.size() == 1,
+  E(args.size() == 1, origin::user,
     F("wrong argument count"));
 
   database db(app);
   project_t project(db);
   packet_writer pw(output);
 
-  revision_id r_id(decode_hexenc(idx(args, 0)()));
-  vector< revision<cert> > certs;
+  revision_id r_id(decode_hexenc_as<revision_id>(idx(args, 0)(), origin::user));
+  vector<cert> certs;
 
-  N(db.revision_exists(r_id), F("no such revision '%s'") % r_id);
+  E(db.revision_exists(r_id), origin::user,
+    F("no such revision '%s'") % r_id);
   project.get_revision_certs(r_id, certs);
 
-  for (vector< revision<cert> >::const_iterator i = certs.begin();
+  for (vector<cert>::const_iterator i = certs.begin();
        i != certs.end(); i++)
     pw.consume_revision_cert(*i);
 }
@@ -1479,17 +1525,18 @@ CMD_AUTOMATE(packet_for_fdata, N_("FILEID"),
              "",
              options::opts::none)
 {
-  N(args.size() == 1,
+  E(args.size() == 1, origin::user,
     F("wrong argument count"));
 
   database db(app);
 
   packet_writer pw(output);
 
-  file_id f_id(decode_hexenc(idx(args, 0)()));
+  file_id f_id(decode_hexenc_as<file_id>(idx(args, 0)(), origin::user));
   file_data f_data;
 
-  N(db.file_version_exists(f_id), F("no such file '%s'") % f_id);
+  E(db.file_version_exists(f_id), origin::user,
+    F("no such file '%s'") % f_id);
   db.get_file_version(f_id, f_data);
   pw.consume_file_data(f_id, f_data);
 }
@@ -1510,20 +1557,20 @@ CMD_AUTOMATE(packet_for_fdelta, N_("OLD_FILE NEW_FILE"),
              "",
              options::opts::none)
 {
-  N(args.size() == 2,
+  E(args.size() == 2, origin::user,
     F("wrong argument count"));
 
   database db(app);
 
   packet_writer pw(output);
 
-  file_id f_old_id(decode_hexenc(idx(args, 0)()));
-  file_id f_new_id(decode_hexenc(idx(args, 1)()));
+  file_id f_old_id(decode_hexenc_as<file_id>(idx(args, 0)(), origin::user));
+  file_id f_new_id(decode_hexenc_as<file_id>(idx(args, 1)(), origin::user));
   file_data f_old_data, f_new_data;
 
-  N(db.file_version_exists(f_old_id),
+  E(db.file_version_exists(f_old_id), origin::user,
     F("no such revision '%s'") % f_old_id);
-  N(db.file_version_exists(f_new_id),
+  E(db.file_version_exists(f_new_id), origin::user,
     F("no such revision '%s'") % f_new_id);
   db.get_file_version(f_old_id, f_old_data);
   db.get_file_version(f_new_id, f_new_data);
@@ -1550,55 +1597,25 @@ CMD_AUTOMATE(common_ancestors, N_("REV1 [REV2 [REV3 [...]]]"),
              "",
              options::opts::none)
 {
-  N(args.size() > 0,
+  E(args.size() > 0, origin::user,
     F("wrong argument count"));
 
   database db(app);
 
-  set<revision_id> ancestors, common_ancestors;
-  vector<revision_id> frontier;
+  set<revision_id> revs, common_ancestors;
   for (args_vector::const_iterator i = args.begin(); i != args.end(); ++i)
     {
-      revision_id rid(decode_hexenc((*i)()));
-      N(db.revision_exists(rid), F("no such revision '%s'") % rid);
-      ancestors.clear();
-      ancestors.insert(rid);
-      frontier.push_back(rid);
-      while (!frontier.empty())
-        {
-          revision_id rid = frontier.back();
-          frontier.pop_back();
-          if(!null_id(rid))
-            {
-              set<revision_id> parents;
-              db.get_revision_parents(rid, parents);
-              for (set<revision_id>::const_iterator i = parents.begin();
-                   i != parents.end(); ++i)
-                {
-                  if (ancestors.find(*i) == ancestors.end())
-                    {
-                      frontier.push_back(*i);
-                      ancestors.insert(*i);
-                    }
-                }
-            }
-        }
-      if (common_ancestors.empty())
-        common_ancestors = ancestors;
-      else
-        {
-          set<revision_id> common;
-          set_intersection(ancestors.begin(), ancestors.end(),
-                         common_ancestors.begin(), common_ancestors.end(),
-                         inserter(common, common.begin()));
-          common_ancestors = common;
-        }
+      revision_id rid(decode_hexenc_as<revision_id>((*i)(), origin::user));
+      E(db.revision_exists(rid), origin::user,
+        F("No such revision %s") % rid);
+      revs.insert(rid);
     }
 
+  db.get_common_ancestors(revs, common_ancestors);
+
   for (set<revision_id>::const_iterator i = common_ancestors.begin();
        i != common_ancestors.end(); ++i)
-    if (!null_id(*i))
-      output << *i << '\n';
+      output << *i << "\n";
 }
 
 // Name: branches
@@ -1618,7 +1635,7 @@ CMD_AUTOMATE(branches, "",
              "",
              options::opts::none)
 {
-  N(args.size() == 0,
+  E(args.size() == 0, origin::user,
     F("no arguments needed"));
 
   database db(app);
@@ -1670,16 +1687,16 @@ CMD_AUTOMATE(tags, N_("[BRANCH_PATTERN]"),
              "",
              options::opts::none)
 {
-  N(args.size() < 2,
+  E(args.size() < 2, origin::user,
     F("wrong argument count"));
 
   database db(app);
   project_t project(db);
-  globish incl("*");
+  globish incl("*", origin::internal);
   bool filtering(false);
 
   if (args.size() == 1) {
-    incl = globish(idx(args, 0)());
+    incl = globish(idx(args, 0)(), origin::user);
     filtering = true;
   }
 
@@ -1769,7 +1786,7 @@ CMD_AUTOMATE(genkey, N_("KEYID PASSPHRASE"),
              "",
              options::opts::none)
 {
-  N(args.size() == 2,
+  E(args.size() == 2, origin::user,
     F("wrong argument count"));
 
   database db(app);
@@ -1818,11 +1835,11 @@ CMD_AUTOMATE(get_option, N_("OPTION"),
              "",
              options::opts::none)
 {
-  N(args.size() == 1,
+  E(args.size() == 1, origin::user,
     F("wrong argument count"));
 
   workspace work(app);
-  work.print_ws_option(args[0], output);
+  work.print_option(args[0], output);
 }
 
 // Name: get_content_changed
@@ -1851,7 +1868,7 @@ CMD_AUTOMATE(get_content_changed, N_("REV FILE"),
              "",
              options::opts::none)
 {
-  N(args.size() == 2,
+  E(args.size() == 2, origin::user,
     F("wrong argument count"));
 
   database db(app);
@@ -1860,13 +1877,13 @@ CMD_AUTOMATE(get_content_changed, N_("REV FILE"),
   revision_id ident;
   marking_map mm;
 
-  ident = revision_id(decode_hexenc(idx(args, 0)()));
-  N(db.revision_exists(ident),
+  ident = decode_hexenc_as<revision_id>(idx(args, 0)(), origin::user);
+  E(db.revision_exists(ident), origin::user,
     F("no revision %s found in database") % ident);
   db.get_roster(ident, new_roster, mm);
 
   file_path path = file_path_external(idx(args,1));
-  N(new_roster.has_node(path),
+  E(new_roster.has_node(path), origin::user,
     F("file %s is unknown for revision %s")
     % path % ident);
 
@@ -1916,7 +1933,7 @@ CMD_AUTOMATE(get_corresponding_path, N_("REV1 FILE REV2"),
              "",
              options::opts::none)
 {
-  N(args.size() == 3,
+  E(args.size() == 3, origin::user,
     F("wrong argument count"));
 
   database db(app);
@@ -1924,18 +1941,18 @@ CMD_AUTOMATE(get_corresponding_path, N_("REV1 FILE REV2"),
   roster_t new_roster, old_roster;
   revision_id ident, old_ident;
 
-  ident = revision_id(decode_hexenc(idx(args, 0)()));
-  N(db.revision_exists(ident),
+  ident = decode_hexenc_as<revision_id>(idx(args, 0)(), origin::user);
+  E(db.revision_exists(ident), origin::user,
     F("no revision %s found in database") % ident);
   db.get_roster(ident, new_roster);
 
-  old_ident = revision_id(decode_hexenc(idx(args, 2)()));
-  N(db.revision_exists(old_ident),
+  old_ident = decode_hexenc_as<revision_id>(idx(args, 2)(), origin::user);
+  E(db.revision_exists(old_ident), origin::user,
     F("no revision %s found in database") % old_ident);
   db.get_roster(old_ident, old_roster);
 
   file_path path = file_path_external(idx(args,1));
-  N(new_roster.has_node(path),
+  E(new_roster.has_node(path), origin::user,
     F("file %s is unknown for revision %s") % path % ident);
 
   node_t node = new_roster.get_node(path);
@@ -1968,7 +1985,7 @@ CMD_AUTOMATE(put_file, N_("[FILEID] CONTENTS"),
              "",
              options::opts::none)
 {
-  N(args.size() == 1 || args.size() == 2,
+  E(args.size() == 1 || args.size() == 2, origin::user,
     F("wrong argument count"));
 
   database db(app);
@@ -1977,17 +1994,17 @@ CMD_AUTOMATE(put_file, N_("[FILEID] CONTENTS"),
   transaction_guard tr(db);
   if (args.size() == 1)
     {
-      file_data dat(idx(args, 0)());
+      file_data dat = typecast_vocab<file_data>(idx(args, 0));
       calculate_ident(dat, sha1sum);
 
       db.put_file(sha1sum, dat);
     }
   else if (args.size() == 2)
     {
-      file_data dat(idx(args, 1)());
+      file_data dat = typecast_vocab<file_data>(idx(args, 1));
       calculate_ident(dat, sha1sum);
-      file_id base_id(decode_hexenc(idx(args, 0)()));
-      N(db.file_version_exists(base_id),
+      file_id base_id(decode_hexenc_as<file_id>(idx(args, 0)(), origin::user));
+      E(db.file_version_exists(base_id), origin::user,
         F("no file version %s found in database") % base_id);
 
       // put_file_version won't do anything if the target ID already exists,
@@ -2023,13 +2040,13 @@ CMD_AUTOMATE(put_revision, N_("REVISION-DATA"),
              "",
              options::opts::none)
 {
-  N(args.size() == 1,
+  E(args.size() == 1, origin::user,
     F("wrong argument count"));
 
   database db(app);
 
   revision_t rev;
-  read_revision(revision_data(idx(args, 0)()), rev);
+  read_revision(typecast_vocab<revision_data>(idx(args, 0)), rev);
 
   // recalculate manifest
   temp_node_id_source nis;
@@ -2060,6 +2077,7 @@ CMD_AUTOMATE(put_revision, N_("REVISION-DATA"),
   // If the database refuses the revision, make sure this is because it's
   // already there.
   E(db.put_revision(id, rev) || db.revision_exists(id),
+    origin::user,
     F("missing prerequisite for revision %s") % id);
 
   output << id << '\n';
@@ -2082,20 +2100,23 @@ CMD_AUTOMATE(cert, N_("REVISION-ID NAME VALUE"),
              "",
              options::opts::none)
 {
-  N(args.size() == 3,
+  E(args.size() == 3, origin::user,
     F("wrong argument count"));
 
   database db(app);
   key_store keys(app);
 
-  hexenc<id> hrid(idx(args, 0)());
-  revision_id rid(decode_hexenc(hrid()));
-  N(db.revision_exists(rid),
+  hexenc<id> hrid(idx(args, 0)(), origin::user);
+  revision_id rid(decode_hexenc_as<revision_id>(hrid(), origin::user));
+  E(db.revision_exists(rid), origin::user,
     F("no such revision '%s'") % hrid);
 
   cache_user_key(app.opts, app.lua, db, keys);
-  put_simple_revision_cert(db, keys, rid, cert_name(idx(args, 1)()),
-                           cert_value(idx(args, 2)()));
+
+  project_t project(db);
+  project.put_cert(keys, rid,
+                   typecast_vocab<cert_name>(idx(args, 1)),
+                   typecast_vocab<cert_value>(idx(args, 2)));
 }
 
 // Name: get_db_variables
@@ -2115,7 +2136,7 @@ CMD_AUTOMATE(get_db_variables, N_("[DOMAIN]"),
              "",
              options::opts::none)
 {
-  N(args.size() < 2,
+  E(args.size() < 2, origin::user,
     F("wrong argument count"));
 
   database db(app);
@@ -2124,7 +2145,7 @@ CMD_AUTOMATE(get_db_variables, N_("[DOMAIN]"),
   if (args.size() == 1)
     {
       filter_by_domain = true;
-      filter = var_domain(idx(args, 0)());
+      filter = typecast_vocab<var_domain>(idx(args, 0));
     }
 
   map<var_key, var_value> vars;
@@ -2146,7 +2167,7 @@ CMD_AUTOMATE(get_db_variables, N_("[DOMAIN]"),
       if (cur_domain != i->first.first)
         {
           // check if we need to print a previous stanza
-          if (st.entries.size() > 0)
+          if (!st.entries.empty())
             {
               pr.print_stanza(st);
               st.entries.clear();
@@ -2158,12 +2179,12 @@ CMD_AUTOMATE(get_db_variables, N_("[DOMAIN]"),
       st.push_str_triple(syms::entry, i->first.second(), i->second());
     }
 
-    N(found_something,
-      F("No variables found or invalid domain specified"));
+  E(found_something, origin::user,
+    F("No variables found or invalid domain specified"));
 
-    // print the last stanza
-    pr.print_stanza(st);
-    output.write(pr.buf.data(), pr.buf.size());
+  // print the last stanza
+  pr.print_stanza(st);
+  output.write(pr.buf.data(), pr.buf.size());
 }
 
 // Name: set_db_variable
@@ -2185,16 +2206,16 @@ CMD_AUTOMATE(set_db_variable, N_("DOMAIN NAME VALUE"),
              "",
              options::opts::none)
 {
-  N(args.size() == 3,
+  E(args.size() == 3, origin::user,
     F("wrong argument count"));
 
   database db(app);
 
-  var_domain domain = var_domain(idx(args, 0)());
+  var_domain domain = typecast_vocab<var_domain>(idx(args, 0));
   utf8 name = idx(args, 1);
   utf8 value = idx(args, 2);
-  var_key key(domain, var_name(name()));
-  db.set_var(key, var_value(value()));
+  var_key key(domain, typecast_vocab<var_name>(name));
+  db.set_var(key, typecast_vocab<var_value>(value));
 }
 
 // Name: drop_db_variables
@@ -2215,18 +2236,18 @@ CMD_AUTOMATE(drop_db_variables, N_("DOMAIN [NAME]"),
              "",
              options::opts::none)
 {
-  N(args.size() == 1 || args.size() == 2,
+  E(args.size() == 1 || args.size() == 2, origin::user,
     F("wrong argument count"));
 
   database db(app);
 
-  var_domain domain(idx(args, 0)());
+  var_domain domain = typecast_vocab<var_domain>(idx(args, 0));
 
   if (args.size() == 2)
     {
-      var_name name(idx(args, 1)());
+      var_name name = typecast_vocab<var_name>(idx(args, 1));
       var_key  key(domain, name);
-      N(db.var_exists(key),
+      E(db.var_exists(key), origin::user,
         F("no var with name %s in domain %s") % name % domain);
       db.clear_var(key);
     }
@@ -2246,11 +2267,77 @@ CMD_AUTOMATE(drop_db_variables, N_("DOMAIN [NAME]"),
             }
         }
 
-      N(found_something,
+      E(found_something, origin::user,
         F("no variables found in domain %s") % domain);
     }
 }
 
+// Name: drop_db_variables
+// Arguments:
+//   none
+// Changes:
+//  8.0 (added)
+// Purpose:
+//   To show the path of the workspace root for the current directory.
+// Output format:
+//   A path
+// Error conditions:
+//   a runtime exception is thrown if the current directory isn't part
+//   of a workspace.
+CMD_AUTOMATE(get_workspace_root, "",
+             N_("Prints the workspace root for the current directory"),
+             "",
+             options::opts::none)
+{
+  workspace work(app);
+  output << get_current_working_dir() << '\n';
+}
+
+// Name: lua
+// Arguments:
+//   A lua function name
+//   Zero or more function arguments
+// Changes:
+//   9.0 (added)
+// Purpose:
+//   Execute lua functions and return their results.
+// Output format:
+//   Lua parsable output.
+// Error conditions:
+//   a runtime exception is thrown if the function does not exists, the arguments cannot be parsed
+//   or the function cannot be executed for some other reason.
+
+CMD_AUTOMATE(lua, "LUA_FUNCTION [ARG1 [ARG2 [...]]]",
+             N_("Executes the given lua function and returns the result"),
+             "",
+             options::opts::none)
+{
+  E(args.size() >= 1, origin::user,
+    F("wrong argument count"));
+
+    std::string func = idx(args, 0)();
+
+    E(app.lua.hook_exists(func), origin::user,
+      F("lua function '%s' does not exist") % func);
+
+    std::vector<std::string> func_args;
+    if (args.size() > 1)
+      {
+        for (unsigned int i=1; i<args.size(); i++)
+        {
+          func_args.push_back(idx(args, i)());
+        }
+      }
+
+    std::string out;
+    E(app.lua.hook_hook_wrapper(func, func_args, out), origin::user,
+      F("lua call '%s' failed") % func);
+
+    // the output already contains a trailing newline, so we don't add
+    // another one here
+    output << out;
+}
+
 // Local Variables:
 // mode: C++
 // fill-column: 76
