@@ -18,6 +18,7 @@
 #include "lua.hh"
 #include "lua_hooks.hh"
 #include "database.hh"
+#include "work.hh"
 
 using std::istream;
 using std::make_pair;
@@ -40,6 +41,10 @@ namespace commands {
                      string const & desc,
                      options::options_type const & opts) :
     command(name, "", CMD_REF(automate), false, false, params, abstract,
+            // We set use_workspace_options true, because all automate
+            // commands need a database, and they expect to get the database
+            // name from the workspace options, even if they don't need a
+            // workspace for anything else.
             desc, true, opts, false)
   {
   }
@@ -63,9 +68,10 @@ namespace commands {
   }
 }
 
-static string const interface_version = "7.0";
-// Major or minor number only increments once for each monotone release;
-// check the most recent release before incrementing this.
+// This number is only raised once, during the process of releasing a new
+// version of monotone, by the release manager. For more details, see
+// point (2) in notes/release-checklist.txt
+static string const interface_version = "9.0";
 
 // Name: interface_version
 // Arguments: none
@@ -82,7 +88,7 @@ CMD_AUTOMATE(interface_version, "",
              "",
              options::opts::none)
 {
-  N(args.size() == 0,
+  E(args.empty(), origin::user,
     F("no arguments needed"));
 
   output << interface_version << '\n';
@@ -154,7 +160,7 @@ class automate_reader
         size = (size*10)+(c-'0');
         read(&c, 1);
       }
-    E(c == ':',
+    E(c == ':', origin::user,
         F("Bad input to automate stdio: expected ':' after string size"));
     char *str = new char[size];
     size_t got = 0;
@@ -174,7 +180,8 @@ class automate_reader
 
     rv = in.rdbuf()->sgetn(buf, nbytes);
 
-    E(eof_ok || rv > 0, F("Bad input to automate stdio: unexpected EOF"));
+    E(eof_ok || rv > 0, origin::user,
+      F("Bad input to automate stdio: unexpected EOF"));
     return rv;
   }
   void go_to_next_item()
@@ -201,7 +208,7 @@ class automate_reader
       case 'o': loc = opt; break;
       case 'l': loc = cmd; break;
       default:
-        E(false,
+        E(false, origin::user,
             F("Bad input to automate stdio: unknown start token '%c'") % c);
       }
   }
@@ -224,14 +231,21 @@ public:
           params.push_back(make_pair(key, val));
         go_to_next_item();
       }
-    E(loc == cmd, F("Bad input to automate stdio: expected '%c' token") % cmd);
+    E(loc == cmd, origin::user,
+      F("Bad input to automate stdio: expected '%c' token") % cmd);
     string item;
     while (get_string(item))
       {
         cmdline.push_back(item);
       }
+    E(cmdline.size() > 0, origin::user,
+        F("Bad input to automate stdio: command name is missing"));
     return true;
   }
+  void reset()
+  {
+    loc = none;
+  }
 };
 
 struct automate_streambuf : public std::streambuf
@@ -338,7 +352,7 @@ CMD_AUTOMATE(stdio, "",
              "",
              options::opts::automate_stdio_size)
 {
-  N(args.size() == 0,
+  E(args.empty(), origin::user,
     F("no arguments needed"));
 
   database db(app);
@@ -347,69 +361,99 @@ CMD_AUTOMATE(stdio, "",
   // immediately if a version discrepancy exists
   db.ensure_open();
 
+  options original_opts = app.opts;
+
   automate_ostream os(output, app.opts.automate_stdio_size);
   automate_reader ar(std::cin);
   vector<pair<string, string> > params;
   vector<string> cmdline;
-  while(ar.get_command(params, cmdline))//while(!EOF)
+  while (true)
     {
+      command const * cmd = 0;
+      command_id id;
       args_vector args;
-      vector<string>::iterator i = cmdline.begin();
-      E(i != cmdline.end(),
-        F("Bad input to automate stdio: command name is missing"));
-      for (; i != cmdline.end(); ++i)
-        {
-          args.push_back(arg_type(*i));
-        }
+
+      // stdio decoding errors should be noted with errno 1,
+      // errno 2 is reserved for errors coming from the commands itself
       try
         {
-          options::options_type opts;
-          opts = options::opts::all_options() - options::opts::globals();
-          opts.instantiate(&app.opts).reset();
+          if (!ar.get_command(params, cmdline))
+            break;
 
-          command_id id;
-          for (args_vector::const_iterator iter = args.begin();
-               iter != args.end(); iter++)
-            id.push_back(utf8((*iter)()));
-
-          if (!id.empty())
+          vector<string>::iterator i = cmdline.begin();
+          for (; i != cmdline.end(); ++i)
             {
-              I(!args.empty());
-
-              set< command_id > matches =
-                CMD_REF(automate)->complete_command(id);
-
-              if (matches.size() == 0)
-                {
-                  N(false, F("no completions for this command"));
-                }
-              else if (matches.size() > 1)
-                {
-                  N(false, F("multiple completions possible for this command"));
-                }
-
-              id = *matches.begin();
-
-              I(args.size() >= id.size());
-              for (command_id::size_type i = 0; i < id.size(); i++)
-                args.erase(args.begin());
-
-              command const * cmd = CMD_REF(automate)->find_command(id);
-              I(cmd != NULL);
-              automate const * acmd = reinterpret_cast< automate const * >(cmd);
-
-              opts = options::opts::globals() | acmd->opts();
-              opts.instantiate(&app.opts).from_key_value_pairs(params);
-              acmd->exec_from_automate(app, id, args, os);
+              args.push_back(arg_type(*i, origin::user));
+              id.push_back(utf8(*i, origin::user));
             }
-          else
-            opts.instantiate(&app.opts).from_key_value_pairs(params);
+
+          set< command_id > matches =
+            CMD_REF(automate)->complete_command(id);
+
+          if (matches.empty())
+            {
+              E(false, origin::user,
+                F("no completions for this command"));
+            }
+          else if (matches.size() > 1)
+            {
+              E(false, origin::user,
+                F("multiple completions possible for this command"));
+            }
+
+          id = *matches.begin();
+
+          I(args.size() >= id.size());
+          for (command_id::size_type i = 0; i < id.size(); i++)
+            args.erase(args.begin());
+
+          cmd = CMD_REF(automate)->find_command(id);
+          I(cmd != NULL);
+
+          if (cmd->use_workspace_options())
+            {
+              // Re-read the ws options file, rather than just copying
+              // the options from the previous apts.opts object, because
+              // the file may have changed due to user activity.
+              workspace::check_format();
+              workspace::get_options(app.opts);
+            }
+
+          options::options_type opts;
+          opts = options::opts::globals() | cmd->opts();
+          opts.instantiate(&app.opts).from_key_value_pairs(params);
         }
-      catch(informative_failure & f)
+      // FIXME: we need to re-package and rethrow this special exception
+      // since it is not based on informative_failure
+      catch (option::option_error & e)
+        {
+          os.set_err(1);
+          os<<e.what();
+          os.end_cmd();
+          ar.reset();
+          continue;
+        }
+      catch (recoverable_failure & f)
+        {
+          os.set_err(1);
+          os<<f.what();
+          os.end_cmd();
+          ar.reset();
+          continue;
+        }
+
+      try
+        {
+          automate const * acmd = dynamic_cast< automate const * >(cmd);
+          I(acmd);
+          acmd->exec_from_automate(app, id, args, os);
+          // set app.opts to the originally given options
+          // so the next command has an identical setup
+          app.opts = original_opts;
+        }
+      catch(recoverable_failure & f)
         {
           os.set_err(2);
-          //Do this instead of printing f.what directly so the output
-          //will be split into properly-sized blocks automatically.
           os<<f.what();
         }
       os.end_cmd();
@@ -425,48 +469,59 @@ LUAEXT(mtn_automate, )
 
   try
     {
-      app_state* app_p = get_app_state(L);
+      app_state* app_p = get_app_state(LS);
       I(app_p != NULL);
-      I(app_p->lua.check_lua_state(L));
-      E(app_p->mtn_automate_allowed,
+      I(app_p->lua.check_lua_state(LS));
+      E(app_p->mtn_automate_allowed, origin::user,
           F("It is illegal to call the mtn_automate() lua extension,\n"
             "unless from a command function defined by register_command()."));
 
       // don't allow recursive calls
       app_p->mtn_automate_allowed = false;
 
-      // automate_ostream os(output, app_p->opts.automate_stdio_size);
+      int n = lua_gettop(LS);
 
-      int n = lua_gettop(L);
-
-      E(n > 0, F("Bad input to mtn_automate() lua extension: command name is missing"));
+      E(n > 0, origin::user,
+        F("Bad input to mtn_automate() lua extension: command name is missing"));
 
       L(FL("Starting call to mtn_automate lua hook"));
 
       for (int i=1; i<=n; i++)
         {
-          arg_type next_arg(luaL_checkstring(L, i));
+          arg_type next_arg(luaL_checkstring(LS, i), origin::user);
           L(FL("arg: %s")%next_arg());
           args.push_back(next_arg);
         }
 
+      options::options_type opts;
+      opts = options::opts::all_options() - options::opts::globals();
+      opts.instantiate(&app_p->opts).reset();
+
+      // the arguments for a command are read from app.opts.args which
+      // is already cleaned from all options. this variable, however, still
+      // contains the original arguments with which the user function was
+      // called. Since we're already in lua context, it makes no sense to
+      // preserve them for the outside world, so we're just clearing them.
+      app_p->opts.args.clear();
+
       commands::command_id id;
       for (args_vector::const_iterator iter = args.begin();
            iter != args.end(); iter++)
-        id.push_back(utf8((*iter)()));
+        id.push_back(*iter);
 
-      E(!id.empty(), F("no command found"));
+      E(!id.empty(), origin::user, F("no command found"));
 
       set< commands::command_id > matches =
         CMD_REF(automate)->complete_command(id);
 
-      if (matches.size() == 0)
+      if (matches.empty())
         {
-          N(false, F("no completions for this command"));
+          E(false, origin::user, F("no completions for this command"));
         }
       else if (matches.size() > 1)
         {
-          N(false, F("multiple completions possible for this command"));
+          E(false, origin::user,
+            F("multiple completions possible for this command"));
         }
 
       id = *matches.begin();
@@ -477,14 +532,29 @@ LUAEXT(mtn_automate, )
 
       commands::command const * cmd = CMD_REF(automate)->find_command(id);
       I(cmd != NULL);
-      commands::automate const * acmd = reinterpret_cast< commands::automate const * >(cmd);
+      opts = options::opts::globals() | cmd->opts();
 
-      acmd->exec(*app_p, id, args, os);
+      if (cmd->use_workspace_options())
+        {
+          // Re-read the ws options file, rather than just copying
+          // the options from the previous apts.opts object, because
+          // the file may have changed due to user activity.
+          workspace::check_format();
+          workspace::get_options(app_p->opts);
+        }
+
+      opts.instantiate(&app_p->opts).from_command_line(args, false);
+      args_vector & parsed_args = app_p->opts.args;
+
+      commands::automate const * acmd
+        = dynamic_cast< commands::automate const * >(cmd);
+      I(acmd);
+      acmd->exec(*app_p, id, app_p->opts.args, os);
 
       // allow further calls
       app_p->mtn_automate_allowed = true;
     }
-  catch(informative_failure & f)
+  catch(recoverable_failure & f)
     {
       // informative failures are passed back to the caller
       result = false;
@@ -499,14 +569,14 @@ LUAEXT(mtn_automate, )
       // invariant failures are permanent
       result = false;
       ui.fatal(e.what());
-      lua_pushstring(L, e.what());
-      lua_error(L);
+      lua_pushstring(LS, e.what());
+      lua_error(LS);
     }
 
   os.flush();
 
-  lua_pushboolean(L, result);
-  lua_pushstring(L, output.str().c_str());
+  lua_pushboolean(LS, result);
+  lua_pushstring(LS, output.str().c_str());
   return 2;
 }
 
