@@ -33,6 +33,7 @@
 #include "vocab_cast.hh"
 #include "app_state.hh"
 #include "project.hh"
+#include "vocab_cast.hh"
 #include "work.hh"
 
 using std::cout;
@@ -69,11 +70,11 @@ CMD(certs, "certs", "", CMD_REF(list), "ID",
 
   revision_id ident;
   complete(app.opts, app.lua,  project, idx(args, 0)(), ident);
-  vector< revision<cert> > ts;
+  vector<cert> ts;
   project.get_revision_certs(ident, ts);
 
   for (size_t i = 0; i < ts.size(); ++i)
-    certs.push_back(idx(ts, i).inner());
+    certs.push_back(idx(ts, i));
 
   {
     set<rsa_keypair_id> checked;
@@ -102,13 +103,13 @@ CMD(certs, "certs", "", CMD_REF(list), "ID",
   if (colon_pos != string::npos)
     {
       string substr(str, 0, colon_pos);
-      colon_pos = display_width(utf8(substr));
+      colon_pos = display_width(utf8(substr, origin::internal));
       extra_str = string(colon_pos, ' ') + ": %s\n";
     }
 
   for (size_t i = 0; i < certs.size(); ++i)
     {
-      cert_status status = check_cert(db, idx(certs, i));
+      cert_status status = db.check_cert(idx(certs, i));
       cert_value tv = idx(certs, i).value;
       string washed;
       if (guess_binary(tv()))
@@ -136,7 +137,7 @@ CMD(certs, "certs", "", CMD_REF(list), "ID",
 
       vector<string> lines;
       split_into_lines(washed, lines);
-      std::string value_first_line = lines.size() > 0 ? idx(lines, 0) : "";
+      std::string value_first_line = lines.empty() ? "" : idx(lines, 0);
 
       cout << string(guess_terminal_width(), '-') << '\n'
            << (i18n_format(str)
@@ -149,12 +150,109 @@ CMD(certs, "certs", "", CMD_REF(list), "ID",
         cout << (i18n_format(extra_str) % idx(lines, i));
     }
 
-  if (certs.size() > 0)
+  if (!certs.empty())
     cout << '\n';
 
   guard.commit();
 }
 
+CMD(duplicates, "duplicates", "", CMD_REF(list), "",
+    N_("Lists duplicate files in the specified revision."
+       " If no revision is specified, use the workspace"),
+    "",
+    options::opts::revision)
+{
+  if (!args.empty())
+    throw usage(execid);
+
+  revision_id rev_id;
+  roster_t roster;
+  database db(app);
+  project_t project(db);
+
+  E(app.opts.revision_selectors.size() <= 1, origin::user,
+    F("more than one revision given"));
+
+  if (app.opts.revision_selectors.empty())
+    {
+      workspace work(app);
+      temp_node_id_source nis;
+
+      work.get_current_roster_shape(db, nis, roster);
+    }
+  else
+    {
+      complete(app.opts, app.lua, project,
+               idx(app.opts.revision_selectors, 0)(), rev_id);
+      E(db.revision_exists(rev_id), origin::user,
+        F("no revision %s found in database") % rev_id);
+      db.get_roster(rev_id, roster);
+    }
+
+  // To find the duplicate files, we put all file_ids in a map
+  // and count how many times they occur in the roster.
+  //
+  // Structure of file_id_map is following:
+  //  first : file_id
+  //  second :
+  //    first : unsigned int
+  //    second : file_paths (=vector<file_path>)
+  typedef std::vector<file_path> file_paths;
+  typedef std::pair<unsigned int, file_paths> file_count;
+  typedef std::map<file_id, file_count> file_id_map;
+  file_id_map file_map;
+
+  node_map const & nodes = roster.all_nodes();
+  for (node_map::const_iterator i = nodes.begin();
+       i != nodes.end(); ++i)
+    {
+      node_t node = i->second;
+      if (is_file_t(node))
+        {
+          file_t f = downcast_to_file_t(node);
+          file_path p;
+          roster.get_name(i->first, p);
+
+          file_id_map::iterator iter = file_map.find(f->content);
+          if (iter == file_map.end())
+            {
+              file_paths paths;
+              paths.push_back(p);
+              file_count fc(1, paths);
+              file_map.insert(make_pair(f->content, fc));
+            }
+          else
+            {
+              iter->second.first++;
+              iter->second.second.push_back(p);
+            }
+        }
+    }
+
+  string empty_checksum(40, ' ');
+  for (file_id_map::const_iterator i = file_map.begin();
+       i != file_map.end(); ++i)
+    {
+      if (i->second.first > 1)
+        {
+          bool first_print = true;
+          for (file_paths::const_iterator j = i->second.second.begin();
+               j != i->second.second.end(); ++j)
+            {
+              if (first_print)
+                {
+                  cout << i->first;
+                  first_print = false;
+                }
+              else
+                cout << empty_checksum;
+
+              cout << " " << *j << '\n';
+            }
+        }
+    }
+}
+
 CMD(keys, "keys", "", CMD_REF(list), "[PATTERN]",
     N_("Lists keys that match a pattern"),
     "",
@@ -165,9 +263,9 @@ CMD(keys, "keys", "", CMD_REF(list), "[PATTERN]",
 
   vector<rsa_keypair_id> pubs;
   vector<rsa_keypair_id> privkeys;
-  globish pattern("*");
+  globish pattern("*", origin::internal);
   if (args.size() == 1)
-    pattern = globish(idx(args, 0)());
+    pattern = globish(idx(args, 0)(), origin::user);
   else if (args.size() > 1)
     throw usage(execid);
 
@@ -205,7 +303,7 @@ CMD(keys, "keys", "", CMD_REF(list), "[PATTERN]",
         }
     }
 
-  if (pubkeys.size() > 0)
+  if (!pubkeys.empty())
     {
       cout << "\n[public keys]\n";
       for (map<rsa_keypair_id, bool>::iterator i = pubkeys.begin();
@@ -236,7 +334,7 @@ CMD(keys, "keys", "", CMD_REF(list), "[PATTERN]",
       cout << '\n';
     }
 
-  if (privkeys.size() > 0)
+  if (!privkeys.empty())
     {
       cout << "\n[private keys]\n";
       for (vector<rsa_keypair_id>::iterator i = privkeys.begin();
@@ -261,10 +359,9 @@ CMD(keys, "keys", "", CMD_REF(list), "[PATTERN]",
         }
     }
 
-  if (pubkeys.size() == 0 &&
-      privkeys.size() == 0)
+  if (pubkeys.empty() && privkeys.empty())
     {
-      if (args.size() == 0)
+      if (args.empty())
         P(F("no keys found"));
       else
         W(F("no keys found matching '%s'") % idx(args, 0)());
@@ -276,9 +373,9 @@ CMD(branches, "branches", "", CMD_REF(list), "[PATTERN]",
     "",
     options::opts::exclude)
 {
-  globish inc("*");
+  globish inc("*", origin::internal);
   if (args.size() == 1)
-    inc = globish(idx(args,0)());
+    inc = globish(idx(args,0)(), origin::user);
   else if (args.size() > 1)
     throw usage(execid);
 
@@ -303,13 +400,15 @@ CMD(epochs, "epochs", "", CMD_REF(list), "[BRANCH [...]]",
   map<branch_name, epoch_data> epochs;
   db.get_epochs(epochs);
 
-  if (args.size() == 0)
+  if (args.empty())
     {
       for (map<branch_name, epoch_data>::const_iterator
              i = epochs.begin();
            i != epochs.end(); ++i)
         {
-          cout << encode_hexenc(i->second.inner()()) << ' ' << i->first << '\n';
+          cout << encode_hexenc(i->second.inner()(),
+                                i->second.inner().made_from)
+               << ' ' << i->first << '\n';
         }
     }
   else
@@ -318,9 +417,12 @@ CMD(epochs, "epochs", "", CMD_REF(list), "[BRANCH [...]]",
            i != args.end();
            ++i)
         {
-          map<branch_name, epoch_data>::const_iterator j = epochs.find(branch_name((*i)()));
-          N(j != epochs.end(), F("no epoch for branch %s") % *i);
-          cout << encode_hexenc(j->second.inner()()) << ' ' << j->first << '\n';
+          map<branch_name, epoch_data>::const_iterator j =
+            epochs.find(typecast_vocab<branch_name>((*i)));
+          E(j != epochs.end(), origin::user, F("no epoch for branch %s") % *i);
+          cout << encode_hexenc(j->second.inner()(),
+                                j->second.inner().made_from)
+               << ' ' << j->first << '\n';
         }
     }
 }
@@ -350,7 +452,7 @@ CMD(vars, "vars", "", CMD_REF(list), "[DOMAIN]",
 {
   bool filterp;
   var_domain filter;
-  if (args.size() == 0)
+  if (args.empty())
     {
       filterp = false;
     }
@@ -390,10 +492,10 @@ CMD(known, "known", "", CMD_REF(list), "",
   temp_node_id_source nis;
   work.get_current_roster_shape(db, nis, new_roster);
 
-  node_restriction mask(work, args_to_paths(args),
+  node_restriction mask(args_to_paths(args),
                         args_to_paths(app.opts.exclude_patterns),
                         app.opts.depth,
-                        new_roster);
+                        new_roster, ignored_file(work));
 
   // to be printed sorted
   vector<file_path> print_paths;
@@ -427,8 +529,8 @@ CMD(unknown, "unknown", "ignored", CMD_REF(list), "",
   workspace work(app);
 
   vector<file_path> roots = args_to_paths(args);
-  path_restriction mask(work, roots, args_to_paths(app.opts.exclude_patterns),
-                        app.opts.depth);
+  path_restriction mask(roots, args_to_paths(app.opts.exclude_patterns),
+                        app.opts.depth, ignored_file(work));
   set<file_path> unknown, ignored;
 
   // if no starting paths have been specified use the workspace root
@@ -459,10 +561,10 @@ CMD(missing, "missing", "", CMD_REF(list), "",
   temp_node_id_source nis;
   roster_t current_roster_shape;
   work.get_current_roster_shape(db, nis, current_roster_shape);
-  node_restriction mask(work, args_to_paths(args),
+  node_restriction mask(args_to_paths(args),
                         args_to_paths(app.opts.exclude_patterns),
                         app.opts.depth,
-                        current_roster_shape);
+                        current_roster_shape, ignored_file(work));
 
   set<file_path> missing;
   work.find_missing(current_roster_shape, mask, missing);
@@ -488,10 +590,10 @@ CMD(changed, "changed", "", CMD_REF(list), "",
 
   work.get_parent_rosters(db, parents);
 
-  node_restriction mask(work, args_to_paths(args),
+  node_restriction mask(args_to_paths(args),
                         args_to_paths(app.opts.exclude_patterns),
                         app.opts.depth,
-                        parents, new_roster);
+                        parents, new_roster, ignored_file(work));
 
   revision_t rrev;
   make_restricted_revision(parents, new_roster, mask, rrev);
@@ -581,7 +683,7 @@ CMD_AUTOMATE(keys, "",
              "",
              options::opts::none)
 {
-  N(args.size() == 0,
+  E(args.empty(), origin::user,
     F("no arguments needed"));
 
   database db(app);
@@ -675,7 +777,7 @@ CMD_AUTOMATE(certs, N_("REV"),
              "",
              options::opts::none)
 {
-  N(args.size() == 1,
+  E(args.size() == 1, origin::user,
     F("wrong argument count"));
 
   database db(app);
@@ -685,18 +787,19 @@ CMD_AUTOMATE(certs, N_("REV"),
 
   transaction_guard guard(db, false);
 
-  hexenc<id> hrid(idx(args, 0)());
-  revision_id rid(decode_hexenc(hrid()));
+  hexenc<id> hrid(idx(args, 0)(), origin::user);
+  revision_id rid(decode_hexenc_as<revision_id>(hrid(), origin::user));
 
-  N(db.revision_exists(rid), F("no such revision '%s'") % hrid);
+  E(db.revision_exists(rid), origin::user,
+    F("no such revision '%s'") % hrid);
 
-  vector< revision<cert> > ts;
+  vector<cert> ts;
   // FIXME_PROJECTS: after projects are implemented,
   // use the db version instead if no project is specified.
   project.get_revision_certs(rid, ts);
 
   for (size_t i = 0; i < ts.size(); ++i)
-    certs.push_back(idx(ts, i).inner());
+    certs.push_back(idx(ts, i));
 
   {
     set<rsa_keypair_id> checked;
@@ -719,7 +822,7 @@ CMD_AUTOMATE(certs, N_("REV"),
   for (size_t i = 0; i < certs.size(); ++i)
     {
       basic_io::stanza st;
-      cert_status status = check_cert(db, idx(certs, i));
+      cert_status status = db.check_cert(idx(certs, i));
       cert_value tv = idx(certs, i).value;
       cert_name name = idx(certs, i).name;
       set<rsa_keypair_id> signers;
@@ -728,7 +831,7 @@ CMD_AUTOMATE(certs, N_("REV"),
       signers.insert(keyid);
 
       bool trusted =
-        app.lua.hook_get_revision_cert_trust(signers, rid,
+        app.lua.hook_get_revision_cert_trust(signers, rid.inner(),
                                              name, tv);
 
       st.push_str_pair(syms::key, keyid());
