@@ -1,14 +1,23 @@
+// Copyright (C) 2002 Graydon Hoare <graydon@pobox.com>
+//               2006 Timothy Brownawell <tbrownaw@gmail.com>
+//
+// This program is made available under the GNU GPL version 2.0 or
+// greater. See the accompanying file COPYING for details.
+//
+// This program is distributed WITHOUT ANY WARRANTY; without even the
+// implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+// PURPOSE.
+
 #include "base.hh"
 #include "cmd.hh"
 
-#include "diff_patch.hh"
+#include "merge_content.hh"
 #include "netcmd.hh"
 #include "globish.hh"
 #include "keys.hh"
 #include "key_store.hh"
 #include "cert.hh"
 #include "revision.hh"
-#include "ui.hh"
 #include "uri.hh"
 #include "vocab_cast.hh"
 #include "platform-wrapped.hh"
@@ -17,6 +26,7 @@
 #include "work.hh"
 #include "database.hh"
 #include "roster.hh"
+#include "vocab_cast.hh"
 
 #include <fstream>
 
@@ -50,10 +60,10 @@ find_key(options & opts,
     return;
 
   rsa_keypair_id key;
-  
+
   utf8 host(info.client.unparsed);
   if (!info.client.u.host.empty())
-    host = utf8(info.client.u.host);
+    host = utf8(info.client.u.host, origin::user);
 
   if (!lua.hook_get_netsync_key(host,
                                 info.client.include_pattern,
@@ -77,28 +87,28 @@ build_client_connection_info(options & opts,
   // Use the default values if needed and available.
   if (!address_given)
     {
-      N(db.var_exists(default_server_key),
+      E(db.var_exists(default_server_key), origin::user,
         F("no server given and no default server set"));
       var_value addr_value;
       db.get_var(default_server_key, addr_value);
-      info.client.unparsed = utf8(addr_value());
+      info.client.unparsed = typecast_vocab<utf8>(addr_value);
       L(FL("using default server address: %s") % info.client.unparsed);
     }
-  parse_uri(info.client.unparsed(), info.client.u);
+  parse_uri(info.client.unparsed(), info.client.u, origin::user);
   if (info.client.u.query.empty() && !include_or_exclude_given)
     {
       // No include/exclude given anywhere, use the defaults.
-      N(db.var_exists(default_include_pattern_key),
+      E(db.var_exists(default_include_pattern_key), origin::user,
         F("no branch pattern given and no default pattern set"));
       var_value pattern_value;
       db.get_var(default_include_pattern_key, pattern_value);
-      info.client.include_pattern = globish(pattern_value());
+      info.client.include_pattern = globish(pattern_value(), origin::user);
       L(FL("using default branch include pattern: '%s'")
         % info.client.include_pattern);
       if (db.var_exists(default_exclude_pattern_key))
         {
           db.get_var(default_exclude_pattern_key, pattern_value);
-          info.client.exclude_pattern = globish(pattern_value());
+          info.client.exclude_pattern = globish(pattern_value(), origin::user);
         }
       else
         info.client.exclude_pattern = globish();
@@ -106,9 +116,9 @@ build_client_connection_info(options & opts,
     }
   else if(!info.client.u.query.empty())
     {
-      N(!include_or_exclude_given,
+      E(!include_or_exclude_given, origin::user,
         F("Include/exclude pattern was given both as part of the URL and as a separate argument."));
-      
+
       // Pull include/exclude from the query string
       char const separator = '/';
       char const negate = '-';
@@ -127,7 +137,7 @@ build_client_connection_info(options & opts,
               if (begin < query.size())
                 end = query.find(separator, begin);
             }
-          
+
           bool is_exclude = false;
           if (item.size() >= 1 && item.at(0) == negate)
             {
@@ -143,21 +153,24 @@ build_client_connection_info(options & opts,
               is_exclude = true;
               item.erase(0, string("exclude=").size());
             }
-          
+
           if (is_exclude)
-            excludes.push_back(arg_type(urldecode(item)));
+            excludes.push_back(arg_type(urldecode(item, origin::user),
+                                        origin::user));
           else
-            includes.push_back(arg_type(urldecode(item)));
+            includes.push_back(arg_type(urldecode(item, origin::user),
+                                        origin::user));
         }
       info.client.include_pattern = globish(includes);
       info.client.exclude_pattern = globish(excludes);
     }
-  
+
   // Maybe set the default values.
   if (!db.var_exists(default_server_key) || opts.set_default)
     {
       P(F("setting default server to %s") % info.client.unparsed());
-      db.set_var(default_server_key, var_value(info.client.unparsed()));
+      db.set_var(default_server_key,
+                 typecast_vocab<var_value>(info.client.unparsed));
     }
     if (!db.var_exists(default_include_pattern_key)
         || opts.set_default)
@@ -165,7 +178,7 @@ build_client_connection_info(options & opts,
         P(F("setting default branch include pattern to '%s'")
           % info.client.include_pattern);
         db.set_var(default_include_pattern_key,
-                   var_value(info.client.include_pattern()));
+                   typecast_vocab<var_value>(info.client.include_pattern));
       }
     if (!db.var_exists(default_exclude_pattern_key)
         || opts.set_default)
@@ -173,9 +186,9 @@ build_client_connection_info(options & opts,
         P(F("setting default branch exclude pattern to '%s'")
           % info.client.exclude_pattern);
         db.set_var(default_exclude_pattern_key,
-                   var_value(info.client.exclude_pattern()));
+                   typecast_vocab<var_value>(info.client.exclude_pattern));
       }
-  
+
   info.client.use_argv =
     lua.hook_get_netsync_connect_command(info.client.u,
                                          info.client.include_pattern,
@@ -207,7 +220,7 @@ extract_client_connection_info(options & opts,
     }
   if (args.size() >= 2 || opts.exclude_given)
     {
-      E(args.size() >= 2, F("no branch pattern given"));
+      E(args.size() >= 2, origin::user, F("no branch pattern given"));
 
       have_include_exclude = true;
       info.client.include_pattern = globish(args.begin() + 1, args.end());
@@ -313,14 +326,14 @@ private:
 };
 
 CMD(clone, "clone", "", CMD_REF(network),
-    N_("ADDRESS[:PORTNUMBER] [DIRECTORY]"),
+    N_("ADDRESS[:PORTNUMBER] BRANCH [DIRECTORY]"),
     N_("Checks out a revision from a remote database into a directory"),
     N_("If a revision is given, that's the one that will be checked out.  "
        "Otherwise, it will be the head of the branch supplied.  "
        "If no directory is given, the branch name will be used as directory"),
-    options::opts::branch | options::opts::revision)
+    options::opts::revision)
 {
-  if (args.size() < 1 || args.size() > 2 || app.opts.revision_selectors.size() > 1)
+  if (args.size() < 2 || args.size() > 3 || app.opts.revision_selectors.size() > 1)
     throw usage(execid);
 
   revision_id ident;
@@ -328,18 +341,20 @@ CMD(clone, "clone", "", CMD_REF(network),
   netsync_connection_info info;
   info.client.unparsed = idx(args, 0);
 
-  N(app.opts.branch_given && !app.opts.branchname().empty(),
+  branch_name branchname = typecast_vocab<branch_name>(idx(args, 1));
+
+  E(!branchname().empty(), origin::user,
     F("you must specify a branch to clone"));
 
-  if (args.size() == 1)
+  if (args.size() == 2)
     {
       // No checkout dir specified, use branch name for dir.
-      workspace_dir = system_path(app.opts.branchname());
+      workspace_dir = system_path(branchname(), origin::user);
     }
   else
     {
       // Checkout to specified dir.
-      workspace_dir = system_path(idx(args, 1));
+      workspace_dir = system_path(idx(args, 2));
     }
 
   require_path_is_nonexistent
@@ -347,7 +362,7 @@ CMD(clone, "clone", "", CMD_REF(network),
 
   // remember the initial working dir so that relative file://
   // db URIs will work
-  system_path start_dir(get_current_working_dir());
+  system_path start_dir(get_current_working_dir(), origin::system);
 
   bool internal_db = !app.opts.dbname_given || app.opts.dbname.empty();
 
@@ -359,9 +374,12 @@ CMD(clone, "clone", "", CMD_REF(network),
                                   / bookkeeping_root_component
                                   / ws_internal_db_file_name);
 
-  // must do this after setting dbname so that _MTN/options is written
-  // correctly
+  // this is actually stupid, but app.opts.branch must be set here
+  // otherwise it will not be written into _MTN/options, in case
+  // a revision is chosen which has multiple branch certs
+  app.opts.branch = branchname;
   workspace::create_workspace(app.opts, app.lua, workspace_dir);
+  app.opts.branch = branch_name();
 
   database db(app);
   if (get_path_status(db.get_filename()) == path::nonexistent)
@@ -372,10 +390,11 @@ CMD(clone, "clone", "", CMD_REF(network),
   key_store keys(app);
   project_t project(db);
 
-  info.client.include_pattern = globish(app.opts.branchname());
+  info.client.include_pattern = globish(branchname(), origin::user);
   info.client.exclude_pattern = globish(app.opts.exclude_patterns);
+
   build_client_connection_info(app.opts, app.lua, db, keys,
-                               info, true, true);
+                               info, true, true, false);
 
   if (app.opts.signing_key() == "")
     P(F("doing anonymous pull; use -kKEYNAME if you need authentication"));
@@ -390,25 +409,21 @@ CMD(clone, "clone", "", CMD_REF(network),
 
   transaction_guard guard(db, false);
 
-  if (app.opts.revision_selectors.size() == 0)
+  if (app.opts.revision_selectors.empty())
     {
-      // use branch head revision
-      N(!app.opts.branchname().empty(),
-        F("use --revision or --branch to specify what to checkout"));
-
       set<revision_id> heads;
-      project.get_branch_heads(app.opts.branchname, heads,
+      project.get_branch_heads(branchname, heads,
                                app.opts.ignore_suspend_certs);
-      N(heads.size() > 0,
-        F("branch '%s' is empty") % app.opts.branchname);
+      E(!heads.empty(), origin::user,
+        F("branch '%s' is empty") % branchname);
       if (heads.size() > 1)
         {
-          P(F("branch %s has multiple heads:") % app.opts.branchname);
+          P(F("branch %s has multiple heads:") % branchname);
           for (set<revision_id>::const_iterator i = heads.begin(); i != heads.end(); ++i)
             P(i18n_format("  %s")
               % describe_revision(project, *i));
-          P(F("choose one with '%s checkout -r<id>'") % ui.prog_name);
-          E(false, F("branch %s has multiple heads") % app.opts.branchname);
+          P(F("choose one with '%s clone -r<id> SERVER BRANCH'") % prog_name);
+          E(false, origin::user, F("branch %s has multiple heads") % branchname);
         }
       ident = *(heads.begin());
     }
@@ -417,12 +432,10 @@ CMD(clone, "clone", "", CMD_REF(network),
       // use specified revision
       complete(app.opts, app.lua, project, idx(app.opts.revision_selectors, 0)(), ident);
 
-      guess_branch(app.opts, project, ident);
-      I(!app.opts.branchname().empty());
-
-      N(project.revision_is_in_branch(ident, app.opts.branchname),
+      E(project.revision_is_in_branch(ident, branchname),
+        origin::user,
         F("revision %s is not a member of branch %s")
-          % ident % app.opts.branchname);
+          % ident % branchname);
     }
 
   roster_t empty_roster, current_roster;
@@ -440,10 +453,8 @@ CMD(clone, "clone", "", CMD_REF(network),
   make_cset(empty_roster, current_roster, checkout);
 
   content_merge_checkout_adaptor wca(db);
-
   work.perform_content_update(db, checkout, wca, false);
 
-  work.update_any_attrs(db);
   work.maybe_update_inodeprints(db);
   guard.commit();
   remove_on_fail.commit();
@@ -458,7 +469,7 @@ struct pid_file
       return;
     require_path_is_nonexistent(path, F("pid file '%s' already exists") % path);
     file.open(path.as_external().c_str());
-    E(file.is_open(), F("failed to create pid file '%s'") % path);
+    E(file.is_open(), origin::system, F("failed to create pid file '%s'") % path);
     file << get_process_id() << '\n';
     file.flush();
   }
@@ -495,18 +506,18 @@ CMD_NO_WORKSPACE(serve, "serve", "", CMD_REF(network), "",
   pid_file pid(app.opts.pidfile);
 
   db.ensure_open();
-  
+
   netsync_connection_info info;
   info.server.addrs = app.opts.bind_uris;
 
   if (app.opts.use_transport_auth)
     {
-      N(app.lua.hook_persist_phrase_ok(),
+      E(app.lua.hook_persist_phrase_ok(), origin::user,
         F("need permission to store persistent passphrase "
           "(see hook persist_phrase_ok())"));
 
-      info.client.include_pattern = globish("*");
-      info.client.exclude_pattern = globish("");
+      info.client.include_pattern = globish("*", origin::internal);
+      info.client.exclude_pattern = globish("", origin::internal);
       if (!app.opts.bind_uris.empty())
         info.client.unparsed = *app.opts.bind_uris.begin();
       find_key(app.opts, app.lua, db, keys, info);
