@@ -16,8 +16,9 @@
 #include <locale.h>
 #include <stdlib.h>
 
-#include "botan/botan.h"
-#include "i18n.h"
+#include <sqlite3.h>
+#include <botan/botan.h>
+
 #include "app_state.hh"
 #include "botan_pipe_cache.hh"
 #include "commands.hh"
@@ -28,12 +29,10 @@
 #include "mt_version.hh"
 #include "option.hh"
 #include "paths.hh"
-#include "sha1.hh"
 #include "simplestring_xform.hh"
 #include "platform.hh"
 #include "work.hh"
 
-
 using std::cout;
 using std::cerr;
 using std::string;
@@ -84,21 +83,6 @@ struct ui_library
   ~ui_library() { ui.deinitialize(); }
 };
 
-// This is in a separate procedure so it can be called from code that's called
-// before cpp_main(), such as program option object creation code.  It's made
-// so it can be called multiple times as well.
-void localize_monotone()
-{
-  static int init = 0;
-  if (!init)
-    {
-      setlocale(LC_ALL, "");
-      bindtextdomain(PACKAGE, get_locale_dir().c_str());
-      textdomain(PACKAGE);
-      init = 1;
-    }
-}
-
 // define the global objects needed by botan_pipe_cache.hh
 pipe_cache_cleanup * global_pipe_cleanup_object;
 Botan::Pipe * unfiltered_pipe;
@@ -110,7 +94,7 @@ read_global_options(options & opts, args_vector & args)
   option::concrete_option_set optset =
     options::opts::all_options().instantiate(&opts);
   optset.from_command_line(args);
-  
+
   return optset;
 }
 
@@ -147,26 +131,47 @@ commands::command_id read_options(options & opts, option::concrete_option_set &
   return cmd;
 }
 
+string
+get_usage_str(options::options_type const & optset, options & opts)
+{
+  vector<string> names;
+  vector<string> descriptions;
+  unsigned int maxnamelen;
+
+  optset.instantiate(&opts).get_usage_strings(names, descriptions, maxnamelen);
+  return format_usage_strings(names, descriptions, maxnamelen);
+}
+
+void
+mtn_terminate_handler()
+{
+  ui.fatal(F("std::terminate() - exception thrown while handling another exception"));
+  exit(3);
+}
+
 int
 cpp_main(int argc, char ** argv)
 {
-  int ret = 0;
-
   // go-go gadget i18n
-  localize_monotone();
+  setlocale(LC_ALL, "");
+  bindtextdomain(PACKAGE, get_locale_dir().c_str());
+  textdomain(PACKAGE);
 
   // set up global ui object - must occur before anything that might try to
   // issue a diagnostic
   ui_library acquire_ui;
 
+  std::set_terminate(&mtn_terminate_handler);
+
   // we want to catch any early informative_failures due to charset
   // conversion etc
   try
     {
       // Set up the global sanity object.  No destructor is needed and
-      // therefore no wrapper object is needed either.
+      // therefore no wrapper object is needed either.  This has the
+      // side effect of making the 'prog_name' global usable.
       global_sanity.initialize(argc, argv, setlocale(LC_ALL, 0));
-      
+
       // Set up secure memory allocation etc
       Botan::LibraryInitializer acquire_botan("thread_safe=0 selftest=0 "
                                               "seed_rng=1 use_engines=0 "
@@ -176,11 +181,11 @@ cpp_main(int argc, char ** argv)
       pipe_cache_cleanup acquire_botan_pipe_caching;
       unfiltered_pipe = new Botan::Pipe;
       new (unfiltered_pipe_cleanup_mem) cached_botan_pipe(unfiltered_pipe);
-      
+
       // Record where we are.  This has to happen before any use of
       // paths.hh objects.
       save_initial_path();
-      
+
       // decode all argv values into a UTF-8 array
       args_vector args;
       for (int i = 1; i < argc; ++i)
@@ -191,17 +196,44 @@ cpp_main(int argc, char ** argv)
           args.push_back(arg_type(ut));
         }
 
-      // find base name of executable, convert to utf8, and save it in the
-      // global ui object
-      {
-        utf8 argv0_u;
-        system_to_utf8(external(argv[0]), argv0_u);
-        string prog_name = system_path(argv0_u).basename()();
-        if (prog_name.rfind(".exe") == prog_name.size() - 4)
-          prog_name = prog_name.substr(0, prog_name.size() - 4);
-        ui.prog_name = prog_name;
-        I(!ui.prog_name.empty());
-      }
+#ifdef SUPPORT_SQLITE_BEFORE_3003014
+      E(sqlite3_libversion_number() >= 3003008, origin::system,
+        F("This monotone binary requires at least SQLite 3.3.8 to run."));
+#else
+      E(sqlite3_libversion_number() >= 3003014, origin::system,
+        F("This monotone binary requires at least SQLite 3.3.14 to run."));
+#endif
+
+      // check the botan library version we got linked against.
+      u32 linked_botan_version = BOTAN_VERSION_CODE_FOR(
+        Botan::version_major(), Botan::version_minor(),
+        Botan::version_patch());
+
+      // Botan 1.7.14 has an incompatible API change, which got reverted
+      // again in 1.7.15. Thus we do not care to support 1.7.14.
+      E(linked_botan_version != BOTAN_VERSION_CODE_FOR(1,7,14), origin::system,
+        F("Monotone does not support Botan 1.7.14."));
+
+#if BOTAN_VERSION_CODE <= BOTAN_VERSION_CODE_FOR(1,7,6)
+      E(linked_botan_version >= BOTAN_VERSION_CODE_FOR(1,6,3), origin::system,
+        F("This monotone binary requires Botan 1.6.3 or newer."));
+      E(linked_botan_version <= BOTAN_VERSION_CODE_FOR(1,7,6), origin::system,
+        F("This monotone binary does not work with Botan newer than 1.7.6."));
+#elif BOTAN_VERSION_CODE <= BOTAN_VERSION_CODE_FOR(1,7,22)
+      E(linked_botan_version > BOTAN_VERSION_CODE_FOR(1,7,6), origin::system,
+        F("This monotone binary requires Botan 1.7.7 or newer."));
+      // While compiling against 1.7.22 or newer is recommended, because
+      // it enables new features of Botan, the monotone binary compiled
+      // against Botan 1.7.21 and before should still work with newer Botan
+      // versions, including all of the stable branch 1.8.x.
+      E(linked_botan_version < BOTAN_VERSION_CODE_FOR(1,9,0), origin::system,
+        F("This monotone binary does not work with Botan 1.9.x."));
+#else
+      E(linked_botan_version > BOTAN_VERSION_CODE_FOR(1,7,22), origin::system,
+        F("This monotone binary requires Botan 1.7.22 or newer."));
+      E(linked_botan_version < BOTAN_VERSION_CODE_FOR(1,9,0), origin::system,
+        F("This monotone binary does not work with Botan 1.9.x."));
+#endif
 
       app_state app;
       try
@@ -209,7 +241,8 @@ cpp_main(int argc, char ** argv)
           // read global options first
           // command specific options will be read below
           args_vector opt_args(args);
-          option::concrete_option_set optset = read_global_options(app.opts, opt_args);
+          option::concrete_option_set optset
+            = read_global_options(app.opts, opt_args);
 
           if (app.opts.version_given)
             {
@@ -221,7 +254,14 @@ cpp_main(int argc, char ** argv)
           // and if found, change directory to it
           // Certain commands may subsequently require a workspace or fail
           // if we didn't find one at this point.
-          workspace::found = find_and_go_to_workspace(app.opts.root);
+          if (app.opts.no_workspace)
+            {
+              workspace::found = false;
+            }
+          else
+            {
+              workspace::found = find_and_go_to_workspace(app.opts.root);
+            }
 
           // Load all available monotonercs.  If we found a workspace above,
           // we'll pick up _MTN/monotonerc as well as the user's monotonerc.
@@ -231,6 +271,12 @@ cpp_main(int argc, char ** argv)
           // this needs to happen after the monotonercs have been read
           commands::command_id cmd = read_options(app.opts, optset, opt_args);
 
+          // check if the user specified default arguments for this command
+          args_vector default_args;
+          if (!cmd.empty()
+              && app.lua.hook_get_default_command_options(cmd, default_args))
+            optset.from_command_line(default_args, false);
+
           if (workspace::found)
             {
               bookkeeping_path dump_path;
@@ -250,28 +296,19 @@ cpp_main(int argc, char ** argv)
 
           // stop here if they asked for help
           if (app.opts.help)
-            {
-              throw usage(cmd);
-            }
+            throw usage(cmd);
 
           // main options processed, now invoke the
           // sub-command w/ remaining args
           if (cmd.empty())
-            {
-              throw usage(commands::command_id());
-            }
-          else
-            {
-              commands::process(app, cmd, app.opts.args);
-              // The command will raise any problems itself through
-              // exceptions.  If we reach this point, it is because it
-              // worked correctly.
-              return 0;
-            }
-        }
-      catch (option::option_error const & e)
-        {
-          N(false, i18n_format("%s") % e.what());
+            throw usage(commands::command_id());
+
+
+          commands::process(app, cmd, app.opts.args);
+          // The command will raise any problems itself through
+          // exceptions.  If we reach this point, it is because it
+          // worked correctly.
+          return 0;
         }
       catch (usage & u)
         {
@@ -287,9 +324,10 @@ cpp_main(int argc, char ** argv)
                                                   u.which.end()))();
 
           usage_stream << F("Usage: %s [OPTION...] command [ARG...]") %
-                          ui.prog_name << "\n\n";
-          usage_stream << options::opts::globals().instantiate(&app.opts).
-                          get_usage_str() << '\n';
+                          prog_name << "\n\n";
+
+          if (u.which.empty())
+            usage_stream << get_usage_str(options::opts::globals(), app.opts);
 
           // Make sure to hide documentation that's not part of
           // the current command.
@@ -297,25 +335,41 @@ cpp_main(int argc, char ** argv)
             commands::command_options(u.which);
           if (!cmd_options.empty())
             {
-              usage_stream << F("Options specific to '%s %s':") %
-                              ui.prog_name % visibleid << "\n\n";
-              usage_stream << cmd_options.instantiate(&app.opts).
-                              get_usage_str() << '\n';
+              usage_stream
+                << F("Options specific to '%s %s' "
+                     "(run '%s help' to see global options):")
+                    % prog_name % visibleid % prog_name
+                << "\n\n";
+              usage_stream << get_usage_str(cmd_options, app.opts);
             }
 
-          commands::explain_usage(u.which, usage_stream);
+          commands::explain_usage(u.which,
+                                  app.opts.show_hidden_commands,
+                                  usage_stream);
           if (app.opts.help)
             return 0;
           else
             return 2;
-
         }
     }
-  catch (informative_failure & inf)
+  catch (option::option_error const & inf)
     {
       ui.inform(inf.what());
       return 1;
     }
+  catch (recoverable_failure & inf)
+    {
+      ui.inform(inf.what());
+      return 1;
+    }
+  catch (unrecoverable_failure & inf)
+    {
+      if (inf.caused_by() == origin::database)
+        ui.fatal_db(inf.what());
+      else
+        ui.fatal(inf.what());
+      return 3;
+    }
   catch (ios_base::failure const & ex)
     {
       // an error has already been printed
