@@ -164,8 +164,8 @@ netcmd::read(string_queue & inbuf, chained_hmac & hmac)
     {
       // they might have given us bogus data
       cmd_digest = extract_substring(inbuf, pos,
-				     constants::netsync_hmac_value_length_in_bytes,
-				     "netcmd HMAC");
+                                     constants::netsync_hmac_value_length_in_bytes,
+                                     "netcmd HMAC");
     }
 
   inbuf.pop_front(pos);
@@ -175,9 +175,9 @@ netcmd::read(string_queue & inbuf, chained_hmac & hmac)
       && cmd_digest != digest)
     {
       throw bad_decode(F("bad HMAC checksum (got %s, wanted %s)\n"
-			 "this suggests data was corrupted in transit")
-		       % encode_hexenc(cmd_digest)
-		       % encode_hexenc(digest));
+                         "this suggests data was corrupted in transit")
+                       % cmd_digest
+                       % digest);
     }
 
   return true;
@@ -215,13 +215,13 @@ netcmd::read_hello_cmd(rsa_keypair_id & server_keyname,
   string skn_str, sk_str;
   extract_variable_length_string(payload, skn_str, pos,
                                  "hello netcmd, server key name");
-  server_keyname = rsa_keypair_id(skn_str);
+  server_keyname = rsa_keypair_id(skn_str, origin::network);
   extract_variable_length_string(payload, sk_str, pos,
                                  "hello netcmd, server key");
-  server_key = rsa_pub_key(sk_str);
+  server_key = rsa_pub_key(sk_str, origin::network);
   nonce = id(extract_substring(payload, pos,
                                constants::merkle_hash_length_in_bytes,
-                               "hello netcmd, nonce"));
+                               "hello netcmd, nonce"), origin::network);
   assert_end_of_buffer(payload, pos, "hello netcmd payload");
 }
 
@@ -275,14 +275,14 @@ netcmd::read_anonymous_cmd(protocol_role & role,
   string pattern_string;
   extract_variable_length_string(payload, pattern_string, pos,
                                  "anonymous(hmac) netcmd, include_pattern");
-  include_pattern = globish(pattern_string);
+  include_pattern = globish(pattern_string, origin::network);
   extract_variable_length_string(payload, pattern_string, pos,
                                  "anonymous(hmac) netcmd, exclude_pattern");
-  exclude_pattern = globish(pattern_string);
+  exclude_pattern = globish(pattern_string, origin::network);
   string hmac_key_string;
   extract_variable_length_string(payload, hmac_key_string, pos,
                                  "anonymous(hmac) netcmd, hmac_key_encrypted");
-  hmac_key_encrypted = rsa_oaep_sha_data(hmac_key_string);
+  hmac_key_encrypted = rsa_oaep_sha_data(hmac_key_string, origin::network);
   assert_end_of_buffer(payload, pos, "anonymous(hmac) netcmd payload");
 }
 
@@ -321,24 +321,26 @@ netcmd::read_auth_cmd(protocol_role & role,
   string pattern_string;
   extract_variable_length_string(payload, pattern_string, pos,
                                  "auth(hmac) netcmd, include_pattern");
-  include_pattern = globish(pattern_string);
+  include_pattern = globish(pattern_string, origin::network);
   extract_variable_length_string(payload, pattern_string, pos,
                                  "auth(hmac) netcmd, exclude_pattern");
-  exclude_pattern = globish(pattern_string);
+  exclude_pattern = globish(pattern_string, origin::network);
   client = id(extract_substring(payload, pos,
                                 constants::merkle_hash_length_in_bytes,
-                                "auth(hmac) netcmd, client identifier"));
+                                "auth(hmac) netcmd, client identifier"),
+              origin::network);
   nonce1 = id(extract_substring(payload, pos,
                                 constants::merkle_hash_length_in_bytes,
-                                "auth(hmac) netcmd, nonce1"));
+                                "auth(hmac) netcmd, nonce1"),
+              origin::network);
   string hmac_key;
   extract_variable_length_string(payload, hmac_key, pos,
                                  "auth(hmac) netcmd, hmac_key_encrypted");
-  hmac_key_encrypted = rsa_oaep_sha_data(hmac_key);
+  hmac_key_encrypted = rsa_oaep_sha_data(hmac_key, origin::network);
   string sig_string;
   extract_variable_length_string(payload, sig_string, pos,
                                  "auth(hmac) netcmd, signature");
-  signature = rsa_sha1_signature(sig_string);
+  signature = rsa_sha1_signature(sig_string, origin::network);
   assert_end_of_buffer(payload, pos, "auth(hmac) netcmd payload");
 }
 
@@ -431,7 +433,8 @@ netcmd::read_data_cmd(netcmd_item_type & type,
   type = read_netcmd_item_type(payload, pos, "data netcmd, item type");
   item = id(extract_substring(payload, pos,
                               constants::merkle_hash_length_in_bytes,
-                              "data netcmd, item identifier"));
+                              "data netcmd, item identifier"),
+            origin::network);
 
   dat.clear();
   u8 compressed_p = extract_datum_lsb<u8>(payload, pos,
@@ -440,7 +443,7 @@ netcmd::read_data_cmd(netcmd_item_type & type,
                                   "data netcmd, data payload");
   if (compressed_p == 1)
   {
-    gzip<data> zdat(dat);
+    gzip<data> zdat(dat, origin::network);
     data tdat;
     decode_gzip(zdat, tdat);
     dat = tdat();
@@ -460,7 +463,7 @@ netcmd::write_data_cmd(netcmd_item_type type,
   if (dat.size() > constants::netcmd_minimum_bytes_to_bother_with_gzip)
     {
       gzip<data> zdat;
-      encode_gzip(data(dat), zdat);
+      encode_gzip(data(dat, origin::internal), zdat);
       payload += static_cast<char>(1); // compressed flag
       insert_variable_length_string(zdat(), payload);
     }
@@ -482,10 +485,12 @@ netcmd::read_delta_cmd(netcmd_item_type & type,
   type = read_netcmd_item_type(payload, pos, "delta netcmd, item type");
   base = id(extract_substring(payload, pos,
                               constants::merkle_hash_length_in_bytes,
-                              "delta netcmd, base identifier"));
+                              "delta netcmd, base identifier"),
+            origin::network);
   ident = id(extract_substring(payload, pos,
                                constants::merkle_hash_length_in_bytes,
-                               "delta netcmd, ident identifier"));
+                               "delta netcmd, ident identifier"),
+             origin::network);
   u8 compressed_p = extract_datum_lsb<u8>(payload, pos,
                                           "delta netcmd, compression flag");
   string tmp;
@@ -493,12 +498,12 @@ netcmd::read_delta_cmd(netcmd_item_type & type,
                                  "delta netcmd, delta payload");
   if (compressed_p == 1)
     {
-      gzip<delta> zdel(tmp);
+      gzip<delta> zdel(tmp, origin::network);
       decode_gzip(zdel, del);
     }
   else
     {
-      del = delta(tmp);
+      del = delta(tmp, origin::network);
     }
   assert_end_of_buffer(payload, pos, "delta netcmd payload");
 }
@@ -539,7 +544,7 @@ netcmd::read_usher_cmd(utf8 & greeting) const
   size_t pos = 0;
   string str;
   extract_variable_length_string(payload, str, pos, "error netcmd, message");
-  greeting = utf8(str);
+  greeting = utf8(str, origin::network);
   assert_end_of_buffer(payload, pos, "error netcmd payload");
 }
 
@@ -553,269 +558,6 @@ netcmd::write_usher_reply_cmd(utf8 const & server, globish const & pattern)
 }
 
 
-#ifdef BUILD_UNIT_TESTS
-
-#include "unit_tests.hh"
-#include "transforms.hh"
-#include "lexical_cast.hh"
-
-UNIT_TEST(netcmd, mac)
-{
-  netcmd out_cmd, in_cmd;
-  string buf;
-  netsync_session_key key(constants::netsync_key_initializer);
-  {
-    chained_hmac mac(key, true);
-    // mutates mac
-    out_cmd.write(buf, mac);
-    UNIT_TEST_CHECK_THROW(in_cmd.read_string(buf, mac), bad_decode);
-  }
-
-  {
-    chained_hmac mac(key, true);
-    out_cmd.write(buf, mac);
-  }
-  buf[0] ^= 0xff;
-  {
-    chained_hmac mac(key, true);
-    UNIT_TEST_CHECK_THROW(in_cmd.read_string(buf, mac), bad_decode);
-  }
-
-  {
-    chained_hmac mac(key, true);
-    out_cmd.write(buf, mac);
-  }
-  buf[buf.size() - 1] ^= 0xff;
-  {
-    chained_hmac mac(key, true);
-    UNIT_TEST_CHECK_THROW(in_cmd.read_string(buf, mac), bad_decode);
-  }
-
-  {
-    chained_hmac mac(key, true);
-    out_cmd.write(buf, mac);
-  }
-  buf += '\0';
-  {
-    chained_hmac mac(key, true);
-    UNIT_TEST_CHECK_THROW(in_cmd.read_string(buf, mac), bad_decode);
-  }
-}
-
-static void
-do_netcmd_roundtrip(netcmd const & out_cmd, netcmd & in_cmd, string & buf)
-{
-  netsync_session_key key(constants::netsync_key_initializer);
-  {
-    chained_hmac mac(key, true);
-    out_cmd.write(buf, mac);
-  }
-  {
-    chained_hmac mac(key, true);
-    UNIT_TEST_CHECK(in_cmd.read_string(buf, mac));
-  }
-  UNIT_TEST_CHECK(in_cmd == out_cmd);
-}
-
-UNIT_TEST(netcmd, functions)
-{
-
-  try
-    {
-
-      // error_cmd
-      {
-        L(FL("checking i/o round trip on error_cmd"));
-        netcmd out_cmd, in_cmd;
-        string out_errmsg("your shoelaces are untied"), in_errmsg;
-        string buf;
-        out_cmd.write_error_cmd(out_errmsg);
-        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_error_cmd(in_errmsg);
-        UNIT_TEST_CHECK(in_errmsg == out_errmsg);
-        L(FL("errmsg_cmd test done, buffer was %d bytes") % buf.size());
-      }
-
-      // hello_cmd
-      {
-        L(FL("checking i/o round trip on hello_cmd"));
-        netcmd out_cmd, in_cmd;
-        string buf;
-        rsa_keypair_id out_server_keyname("server@there"), in_server_keyname;
-        rsa_pub_key out_server_key("9387938749238792874"), in_server_key;
-        id out_nonce(raw_sha1("nonce it up")), in_nonce;
-        out_cmd.write_hello_cmd(out_server_keyname, out_server_key, out_nonce);
-        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_hello_cmd(in_server_keyname, in_server_key, in_nonce);
-        UNIT_TEST_CHECK(in_server_keyname == out_server_keyname);
-        UNIT_TEST_CHECK(in_server_key == out_server_key);
-        UNIT_TEST_CHECK(in_nonce == out_nonce);
-        L(FL("hello_cmd test done, buffer was %d bytes") % buf.size());
-      }
-
-      // bye_cmd
-      {
-        L(FL("checking i/o round trip on bye_cmd"));
-        netcmd out_cmd, in_cmd;
-        u8 out_phase(1), in_phase;
-        string buf;
-
-        out_cmd.write_bye_cmd(out_phase);
-        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_bye_cmd(in_phase);
-        UNIT_TEST_CHECK(in_phase == out_phase);
-        L(FL("bye_cmd test done, buffer was %d bytes") % buf.size());
-      }
-
-      // anonymous_cmd
-      {
-        L(FL("checking i/o round trip on anonymous_cmd"));
-        netcmd out_cmd, in_cmd;
-        protocol_role out_role = source_and_sink_role, in_role;
-        string buf;
-        // total cheat, since we don't actually verify that rsa_oaep_sha_data
-        // is sensible anywhere here...
-        rsa_oaep_sha_data out_key("nonce start my heart"), in_key;
-        globish out_include_pattern("radishes galore!"), in_include_pattern;
-        globish out_exclude_pattern("turnips galore!"), in_exclude_pattern;
-
-        out_cmd.write_anonymous_cmd(out_role, out_include_pattern, out_exclude_pattern, out_key);
-        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_anonymous_cmd(in_role, in_include_pattern, in_exclude_pattern, in_key);
-        UNIT_TEST_CHECK(in_key == out_key);
-        UNIT_TEST_CHECK(in_include_pattern() == out_include_pattern());
-        UNIT_TEST_CHECK(in_exclude_pattern() == out_exclude_pattern());
-        UNIT_TEST_CHECK(in_role == out_role);
-        L(FL("anonymous_cmd test done, buffer was %d bytes") % buf.size());
-      }
-
-      // auth_cmd
-      {
-        L(FL("checking i/o round trip on auth_cmd"));
-        netcmd out_cmd, in_cmd;
-        protocol_role out_role = source_and_sink_role, in_role;
-        string buf;
-        id out_client(raw_sha1("happy client day")), out_nonce1(raw_sha1("nonce me amadeus")),
-          in_client, in_nonce1;
-        // total cheat, since we don't actually verify that rsa_oaep_sha_data
-        // is sensible anywhere here...
-        rsa_oaep_sha_data out_key("nonce start my heart"), in_key;
-        rsa_sha1_signature out_signature(raw_sha1("burble") + raw_sha1("gorby")), in_signature;
-        globish out_include_pattern("radishes galore!"), in_include_pattern;
-        globish out_exclude_pattern("turnips galore!"), in_exclude_pattern;
-
-        out_cmd.write_auth_cmd(out_role, out_include_pattern, out_exclude_pattern
-                               , out_client, out_nonce1, out_key, out_signature);
-        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_auth_cmd(in_role, in_include_pattern, in_exclude_pattern,
-                             in_client, in_nonce1, in_key, in_signature);
-        UNIT_TEST_CHECK(in_client == out_client);
-        UNIT_TEST_CHECK(in_nonce1 == out_nonce1);
-        UNIT_TEST_CHECK(in_key == out_key);
-        UNIT_TEST_CHECK(in_signature == out_signature);
-        UNIT_TEST_CHECK(in_role == out_role);
-        UNIT_TEST_CHECK(in_include_pattern() == out_include_pattern());
-        UNIT_TEST_CHECK(in_exclude_pattern() == out_exclude_pattern());
-        L(FL("auth_cmd test done, buffer was %d bytes") % buf.size());
-      }
-
-      // confirm_cmd
-      {
-        L(FL("checking i/o round trip on confirm_cmd"));
-        netcmd out_cmd, in_cmd;
-        string buf;
-        out_cmd.write_confirm_cmd();
-        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_confirm_cmd();
-        L(FL("confirm_cmd test done, buffer was %d bytes") % buf.size());
-      }
-
-      // refine_cmd
-      {
-        L(FL("checking i/o round trip on refine_cmd"));
-        netcmd out_cmd, in_cmd;
-        string buf;
-        refinement_type out_ty (refinement_query), in_ty(refinement_response);
-        merkle_node out_node, in_node;
-
-        out_node.set_raw_slot(0, id(raw_sha1("The police pulled Kris Kringle over")));
-        out_node.set_raw_slot(3, id(raw_sha1("Kris Kringle tried to escape from the police")));
-        out_node.set_raw_slot(8, id(raw_sha1("He was arrested for auto theft")));
-        out_node.set_raw_slot(15, id(raw_sha1("He was whisked away to jail")));
-        out_node.set_slot_state(0, subtree_state);
-        out_node.set_slot_state(3, leaf_state);
-        out_node.set_slot_state(8, leaf_state);
-        out_node.set_slot_state(15, subtree_state);
-
-        out_cmd.write_refine_cmd(out_ty, out_node);
-        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_refine_cmd(in_ty, in_node);
-        UNIT_TEST_CHECK(in_ty == out_ty);
-        UNIT_TEST_CHECK(in_node == out_node);
-        L(FL("refine_cmd test done, buffer was %d bytes") % buf.size());
-      }
-
-      // done_cmd
-      {
-        L(FL("checking i/o round trip on done_cmd"));
-        netcmd out_cmd, in_cmd;
-        size_t out_n_items(12), in_n_items(0);
-        netcmd_item_type out_type(key_item), in_type(revision_item);
-        string buf;
-
-        out_cmd.write_done_cmd(out_type, out_n_items);
-        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_done_cmd(in_type, in_n_items);
-        UNIT_TEST_CHECK(in_n_items == out_n_items);
-        UNIT_TEST_CHECK(in_type == out_type);
-        L(FL("done_cmd test done, buffer was %d bytes") % buf.size());
-      }
-
-      // data_cmd
-      {
-        L(FL("checking i/o round trip on data_cmd"));
-        netcmd out_cmd, in_cmd;
-        netcmd_item_type out_type(file_item), in_type(key_item);
-        id out_id(raw_sha1("tuna is not yummy")), in_id;
-        string out_dat("thank you for flying northwest"), in_dat;
-        string buf;
-        out_cmd.write_data_cmd(out_type, out_id, out_dat);
-        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_data_cmd(in_type, in_id, in_dat);
-        UNIT_TEST_CHECK(in_id == out_id);
-        UNIT_TEST_CHECK(in_dat == out_dat);
-        L(FL("data_cmd test done, buffer was %d bytes") % buf.size());
-      }
-
-      // delta_cmd
-      {
-        L(FL("checking i/o round trip on delta_cmd"));
-        netcmd out_cmd, in_cmd;
-        netcmd_item_type out_type(file_item), in_type(key_item);
-        id out_head(raw_sha1("your seat cusion can be reused")), in_head;
-        id out_base(raw_sha1("as a floatation device")), in_base;
-        delta out_delta("goodness, this is not an xdelta"), in_delta;
-        string buf;
-
-        out_cmd.write_delta_cmd(out_type, out_head, out_base, out_delta);
-        do_netcmd_roundtrip(out_cmd, in_cmd, buf);
-        in_cmd.read_delta_cmd(in_type, in_head, in_base, in_delta);
-        UNIT_TEST_CHECK(in_type == out_type);
-        UNIT_TEST_CHECK(in_head == out_head);
-        UNIT_TEST_CHECK(in_base == out_base);
-        UNIT_TEST_CHECK(in_delta == out_delta);
-        L(FL("delta_cmd test done, buffer was %d bytes") % buf.size());
-      }
-
-    }
-  catch (bad_decode & d)
-    {
-      L(FL("bad decode exception: '%s'") % d.what);
-      throw;
-    }
-}
-
-#endif // BUILD_UNIT_TESTS
 
 // Local Variables:
 // mode: C++
