@@ -42,7 +42,9 @@ enum selector_type
     sel_cert,
     sel_earlier,
     sel_later,
+    sel_message,
     sel_parent,
+    sel_update,
     sel_unknown
   };
 
@@ -105,9 +107,15 @@ decode_selector(project_t & project,
         case 'e':
           type = sel_earlier;
           break;
+        case 'm':
+          type = sel_message;
+          break;
         case 'p':
           type = sel_parent;
           break;
+        case 'u':
+          type = sel_update;
+          break;
         default:
           W(F("unknown selector type: %c") % sel[0]);
           break;
@@ -121,8 +129,8 @@ decode_selector(project_t & project,
         case sel_later:
         case sel_earlier:
           if (lua.hook_exists("expand_date"))
-            { 
-              N(lua.hook_expand_date(sel, tmp),
+            {
+              E(lua.hook_expand_date(sel, tmp), origin::user,
                 F("selector '%s' is not a valid date\n") % sel);
             }
           else
@@ -140,7 +148,7 @@ decode_selector(project_t & project,
             tmp += "-01T00:00:00";
           else if (tmp.size()<11 && (sel_later==type || sel_earlier==type))
             tmp += "T00:00:00";
-          N(tmp.size()==19 || sel_date==type, 
+          E(tmp.size()==19 || sel_date==type, origin::user,
             F("selector '%s' is not a valid date (%s)") % sel % tmp);
 
           if (sel != tmp)
@@ -163,12 +171,12 @@ decode_selector(project_t & project,
                 : F("the empty head selector h: refers to "
                     "the head of the current branch");
               workspace::require_workspace(msg);
-              sel = opts.branchname();
+              sel = opts.branch();
             }
           break;
 
         case sel_cert:
-          N(!sel.empty(),
+          E(!sel.empty(), origin::user,
             F("the cert selector c: may not be empty"));
           break;
 
@@ -190,17 +198,29 @@ decode_selector(project_t & project,
                 }
 
               diagnose_ambiguous_expansion(project, "p:", parent_ids);
-              sel = (* parent_ids.begin()).inner()();
+              sel = encode_hexenc((* parent_ids.begin()).inner()(),
+                                  origin::internal);
             }
-          else
-            sel = decode_hexenc(sel);
           break;
+        case sel_update:
+          E(sel.empty(), origin::user,
+            F("no value is allowed with the update selector u:"));
+          {
+            workspace work(opts, lua, F("the update selector u: refers to the "
+                                        "revision before the last update in the "
+                                        "workspace"));
+            revision_id update_id;
+            work.get_update_id(update_id);
+            sel = encode_hexenc(update_id.inner()(), origin::internal);
+          }
+          break;
+
         default: break;
         }
     }
 }
 
-static void 
+static void
 parse_selector(project_t & project,
                options const & opts,
                lua_hooks & lua,
@@ -251,7 +271,11 @@ complete_one_selector(project_t & project,
       I(!value.empty());
       project.db.select_parent(value, completions);
       break;
-        
+
+    case sel_update:
+      project.db.complete(value, completions);
+      break;
+
     case sel_author:
       project.db.select_cert(author_cert_name(), value, completions);
       break;
@@ -281,6 +305,16 @@ complete_one_selector(project_t & project,
       project.db.select_date(value, ">", completions);
       break;
 
+    case sel_message:
+      {
+        set<revision_id> changelogs, comments;
+        project.db.select_cert(changelog_cert_name(), value, changelogs);
+        project.db.select_cert(comment_cert_name(), value, comments);
+        completions.insert(changelogs.begin(), changelogs.end());
+        completions.insert(comments.begin(), comments.end());
+      }
+      break;
+
     case sel_cert:
       {
         I(!value.empty());
@@ -308,7 +342,7 @@ complete_one_selector(project_t & project,
         // get branch names
         set<branch_name> branch_names;
         I(!value.empty());
-        project.get_branch_list(globish(value), branch_names);
+        project.get_branch_list(globish(value, origin::user), branch_names);
 
         L(FL("found %d matching branches") % branch_names.size());
 
@@ -372,8 +406,8 @@ complete(options const & opts, lua_hooks & lua,
       && sels[0].first == sel_ident
       && sels[0].second.size() == constants::idlen)
     {
-      completions.insert(revision_id(decode_hexenc(sels[0].second)));
-      N(project.db.revision_exists(*completions.begin()),
+      completions.insert(decode_hexenc_as<revision_id>(sels[0].second, origin::user));
+      E(project.db.revision_exists(*completions.begin()), origin::user,
         F("no such revision '%s'") % *completions.begin());
       return;
     }
@@ -381,7 +415,7 @@ complete(options const & opts, lua_hooks & lua,
   P(F("expanding selection '%s'") % str);
   complete_selector(project, sels, completions);
 
-  N(completions.size() != 0,
+  E(!completions.empty(), origin::user,
     F("no match for selection '%s'") % str);
 
   for (set<revision_id>::const_iterator i = completions.begin();
@@ -391,7 +425,7 @@ complete(options const & opts, lua_hooks & lua,
 
       // This may be impossible, but let's make sure.
       // All the callers used to do it.
-      N(project.db.revision_exists(*i),
+      E(project.db.revision_exists(*i), origin::user,
         F("no such revision '%s'") % *i);
     }
 }
@@ -406,7 +440,7 @@ complete(options const & opts, lua_hooks & lua,
 
   complete(opts, lua, project, str, completions);
 
-  I(completions.size() > 0);
+  I(!completions.empty());
   diagnose_ambiguous_expansion(project, str, completions);
 
   completion = *completions.begin();
@@ -427,7 +461,8 @@ expand_selector(options const & opts, lua_hooks & lua,
       && sels[0].first == sel_ident
       && sels[0].second.size() == constants::idlen)
     {
-      completions.insert(revision_id(decode_hexenc(sels[0].second)));
+      completions.insert(decode_hexenc_as<revision_id>(sels[0].second,
+                                                       origin::user));
       return;
     }
 
@@ -448,7 +483,7 @@ diagnose_ambiguous_expansion(project_t & project,
        i != completions.end(); ++i)
     err += ("\n" + describe_revision(project, *i));
 
-  N(false, i18n_format(err));
+  E(false, origin::user, i18n_format(err));
 }
 
 
