@@ -1,3 +1,11 @@
+-- Copyright (C) 2003 Graydon Hoare <graydon@pobox.com>
+--
+-- This program is made available under the GNU GPL version 2.0 or
+-- greater. See the accompanying file COPYING for details.
+--
+-- This program is distributed WITHOUT ANY WARRANTY; without even the
+-- implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+-- PURPOSE.
 
 -- this is the standard set of lua hooks for monotone;
 -- user-provided files can override it or add to it.
@@ -88,7 +96,9 @@ end
 attr_functions["mtn:execute"] =
    function(filename, value)
       if (value == "true") then
-         make_executable(filename)
+         set_executable(filename)
+      else
+         clear_executable(filename)
       end
    end
 
@@ -102,18 +112,26 @@ function dir_matches(name, dir)
    return false
 end
 
+function portable_readline(f)
+    line = f:read()
+    if line ~= nil then
+        line = string.gsub(line, "\r$","") -- strip possible \r left from windows editing
+    end
+    return line
+end
+
 function ignore_file(name)
    -- project specific
    if (ignored_files == nil) then
       ignored_files = {}
       local ignfile = io.open(".mtn-ignore", "r")
       if (ignfile ~= nil) then
-         local line = ignfile:read()
+         local line = portable_readline(ignfile)
          while (line ~= nil) do
             if line ~= "" then
                 table.insert(ignored_files, line)
             end
-            line = ignfile:read()
+            line = portable_readline(ignfile)
          end
          io.close(ignfile)
       end
@@ -123,23 +141,25 @@ function ignore_file(name)
    for i, line in pairs(ignored_files)
    do
       if (line ~= nil) then
-         local pcallstatus, result = pcall(function() 
-	    return regex.search(line, name) 
-	 end)
+         local pcallstatus, result = pcall(function()
+        return regex.search(line, name)
+     end)
          if pcallstatus == true then
             -- no error from the regex.search call
             if result == true then return true end
          else
-            -- regex.search had a problem, warn the user their 
+            -- regex.search had a problem, warn the user their
             -- .mtn-ignore file syntax is wrong
-	    if not warn_reported_file then
-	       io.stderr:write("mtn: warning: while matching file '"
-	       		       .. name .. "':\n")
-	       warn_reported_file = true
-	    end
-            io.stderr:write(".mtn-ignore:" .. i .. ": warning: " .. result
-	    		    .. "\n\t- skipping this regex for "
-			    .. "all remaining files.\n")
+        if not warn_reported_file then
+           io.stderr:write("mtn: warning: while matching file '"
+                       .. name .. "':\n")
+           warn_reported_file = true
+        end
+        local prefix = ".mtn-ignore:" .. i .. ": warning: "
+            io.stderr:write(prefix
+                            .. string.gsub(result, "\n", "\n" .. prefix)
+                               .. "\n\t- skipping this regex for "
+                               .. "all remaining files.\n")
             ignored_files[i] = nil
          end
       end
@@ -259,21 +279,21 @@ end
 
 function edit_comment(basetext, user_log_message)
    local exe = nil
-   if (program_exists_in_path("vi")) then exe = "vi" end
-   if (string.sub(get_ostype(), 1, 6) ~= "CYGWIN" and program_exists_in_path("notepad.exe")) then exe = "notepad.exe" end
-   local debian_editor = io.open("/usr/bin/editor")
-   if (debian_editor ~= nil) then
-      debian_editor:close()
-      exe = "/usr/bin/editor"
-   end
-   local visual = os.getenv("VISUAL")
-   if (visual ~= nil) then exe = visual end
-   local editor = os.getenv("EDITOR")
-   if (editor ~= nil) then exe = editor end
 
-   if (exe == nil) then
-      io.write("Could not find editor to enter commit message\n"
-               .. "Try setting the environment variable EDITOR\n")
+   -- top priority is VISUAL, then EDITOR, then a series of hardcoded
+   -- defaults, if available.
+
+   local visual = os.getenv("VISUAL")
+   local editor = os.getenv("EDITOR")
+   if (visual ~= nil) then exe = visual
+   elseif (editor ~= nil) then exe = editor
+   elseif (program_exists_in_path("editor")) then exe = "editor"
+   elseif (program_exists_in_path("vi")) then exe = "vi"
+   elseif (string.sub(get_ostype(), 1, 6) ~= "CYGWIN" and
+	   program_exists_in_path("notepad.exe")) then exe = "notepad"
+   else
+      io.write(gettext("Could not find editor to enter commit message\n"
+		       .. "Try setting the environment variable EDITOR\n"))
       return nil
    end
 
@@ -287,11 +307,44 @@ function edit_comment(basetext, user_log_message)
    tmp:write(basetext)
    io.close(tmp)
 
-   if (execute(exe, tname) ~= 0) then
-      io.write(string.format(gettext("Error running editor '%s' to enter log message\n"),
-                             exe))
-      os.remove(tname)
-      return nil
+   -- By historical convention, VISUAL and EDITOR can contain arguments
+   -- (and, in fact, arbitrarily complicated shell constructs).  Since Lua
+   -- has no word-splitting functionality, we invoke the shell to deal with
+   -- anything more complicated than a single word with no metacharacters.
+   -- This, unfortunately, means we have to quote the file argument.
+
+   if (not string.find(exe, "[^%w_.+-]")) then
+      -- safe to call spawn directly
+      if (execute(exe, tname) ~= 0) then
+	 io.write(string.format(gettext("Error running editor '%s' "..
+					"to enter log message\n"),
+                                exe))
+	 os.remove(tname)
+	 return nil
+      end
+   else
+      -- must use shell
+      local shell = os.getenv("SHELL")
+      if (shell == nil) then shell = "sh" end
+      if (not program_exists_in_path(shell)) then
+	 io.write(string.format(gettext("Editor command '%s' needs a shell, "..
+					"but '%s' is not to be found"),
+			        exe, shell))
+	 os.remove(tname)
+	 return nil
+      end
+
+      -- Single-quoted strings in both Bourne shell and csh can contain
+      -- anything but a single quote.
+      local safe_tname = " '" .. string.gsub(tname, "'", "'\\''") .. "'"
+
+      if (execute(shell, "-c", editor .. safe_tname) ~= 0) then
+	 io.write(string.format(gettext("Error running editor '%s' "..
+					"to enter log message\n"),
+                                exe))
+	 os.remove(tname)
+	 return nil
+      end
    end
 
    tmp = io.open(tname, "r")
@@ -424,27 +477,27 @@ mergers.tortoise = {
 mergers.vim = {
    cmd = function (tbl)
       function execute_diff3(mine, yours, out)
-	 local diff3_args = {
-	    "diff3",
-	    "--merge",
-	    "--easy-only",
-	 }
-	 table.insert(diff3_args, string.gsub(mine, "\\", "/") .. "")
-	 table.insert(diff3_args, string.gsub(tbl.afile, "\\", "/") .. "")
-	 table.insert(diff3_args, string.gsub(yours, "\\", "/") .. "")
-      
-	 return execute_redirected("", string.gsub(out, "\\", "/"), "", unpack(diff3_args))
+     local diff3_args = {
+        "diff3",
+        "--merge",
+        "--easy-only",
+     }
+     table.insert(diff3_args, string.gsub(mine, "\\", "/") .. "")
+     table.insert(diff3_args, string.gsub(tbl.afile, "\\", "/") .. "")
+     table.insert(diff3_args, string.gsub(yours, "\\", "/") .. "")
+
+     return execute_redirected("", string.gsub(out, "\\", "/"), "", unpack(diff3_args))
       end
 
       io.write (string.format("\nWARNING: 'vim' was choosen to perform external 3-way merge.\n"..
           "You should merge all changes to *LEFT* file due to limitation of program\n"..
           "arguments.\n\n"))
-      
+
       local vim
       if os.getenv ("DISPLAY") ~= nil and program_exists_in_path ("gvim") then
-	 vim = "gvim"
+     vim = "gvim"
       else
-	 vim = "vim"
+     vim = "vim"
       end
 
       local lfile_merged = tbl.lfile .. ".merged"
@@ -455,7 +508,7 @@ mergers.vim = {
       if ret == 2 then
          io.write(string.format(gettext("Error running diff3 for merger '%s'\n"), vim))
          os.remove(lfile_merged)
-	 return false
+     return false
       end
 
       -- now merge rfile using diff3
@@ -464,13 +517,13 @@ mergers.vim = {
          io.write(string.format(gettext("Error running diff3 for merger '%s'\n"), vim))
          os.remove(lfile_merged)
          os.remove(rfile_merged)
-	 return false
+     return false
       end
-      
+
       os.rename(lfile_merged, tbl.lfile)
       os.rename(rfile_merged, tbl.rfile)
-      
-      local ret = execute(vim, "-f", "-d", "-c", string.format("file %s", tbl.outfile),
+
+      local ret = execute(vim, "-f", "-d", "-c", string.format("silent file %s", tbl.outfile),
                           tbl.lfile, tbl.rfile)
       if (ret ~= 0) then
          io.write(string.format(gettext("Error running merger '%s'\n"), vim))
@@ -480,19 +533,19 @@ mergers.vim = {
    end ,
    available =
       function ()
-	 return program_exists_in_path("diff3") and
+     return program_exists_in_path("diff3") and
             (program_exists_in_path("vim") or
-	    program_exists_in_path("gvim"))
+        program_exists_in_path("gvim"))
       end ,
    wanted =
       function ()
-	 local editor = os.getenv("EDITOR")
-	 if editor and
-	    not (string.find(editor, "vim") or
-		 string.find(editor, "gvim")) then
-	    return false
-	 end
-	 return true
+     local editor = os.getenv("EDITOR")
+     if editor and
+        not (string.find(editor, "vim") or
+         string.find(editor, "gvim")) then
+        return false
+     end
+     return true
       end
 }
 
@@ -518,9 +571,9 @@ mergers.rcsmerge = {
    end,
    available =
       function ()
-	 local merge = os.getenv("MTN_RCSMERGE")
-	 return merge and
-	    program_exists_in_path(merge) and program_exists_in_path("vim")
+     local merge = os.getenv("MTN_RCSMERGE")
+     return merge and
+        program_exists_in_path(merge) and program_exists_in_path("vim")
       end ,
    wanted = function () return os.getenv("MTN_RCSMERGE") ~= nil end
 }
@@ -631,7 +684,7 @@ mergers.diffutils = {
         --  assume it is requested (if it is available at all)
         return true
     end
-}   
+}
 
 mergers.emacs = {
    cmd = function (tbl)
@@ -658,18 +711,18 @@ mergers.emacs = {
    end,
    available =
       function ()
-	 return program_exists_in_path("xemacs") or
-	    program_exists_in_path("emacs")
+     return program_exists_in_path("xemacs") or
+        program_exists_in_path("emacs")
       end ,
    wanted =
       function ()
-	 local editor = os.getenv("EDITOR")
-	 if editor and
-	    not (string.find(editor, "emacs") or
-		 string.find(editor, "gnu")) then
-	    return false
-	 end
-	 return true
+     local editor = os.getenv("EDITOR")
+     if editor and
+        not (string.find(editor, "emacs") or
+         string.find(editor, "gnu")) then
+        return false
+     end
+     return true
       end
 }
 
@@ -841,17 +894,17 @@ function merge3 (anc_path, left_path, right_path, merged_path, ancestor, left, r
             end
          end
       else
-	 if mkey then
-	    io.write (string.format("The possible commands for the "..mkey.." merger aren't available.\n"..
+     if mkey then
+        io.write (string.format("The possible commands for the "..mkey.." merger aren't available.\n"..
                 "You may want to check that $MTN_MERGE or the lua variable `merger' is set\n"..
                 "to something available.  If you want to use vim or emacs, you can also\n"..
-		"set $EDITOR to something appropriate.\n"))
-	 else
-	    io.write (string.format("No external 3-way merge command found.\n"..
+        "set $EDITOR to something appropriate.\n"))
+     else
+        io.write (string.format("No external 3-way merge command found.\n"..
                 "You may want to check that $EDITOR is set to an editor that supports 3-way\n"..
                 "merge, set this explicitly in your get_preferred_merge3_command hook,\n"..
                 "or add a 3-way merge program to your path.\n"))
-	 end
+     end
       end
    end
 
@@ -1144,6 +1197,59 @@ function get_mtn_command(host)
         return "mtn"
 end
 
+function get_default_command_options(command)
+   local default_args = {}
+   return default_args
+end
+
+hook_wrapper_dump                = {}
+hook_wrapper_dump.depth          = 0
+hook_wrapper_dump._string        = function(s) return string.format("%q", s) end
+hook_wrapper_dump._number        = function(n) return tostring(n) end
+hook_wrapper_dump._boolean       = function(b) if (b) then return "true" end return "false" end
+hook_wrapper_dump._userdata      = function(u) return "nil --[[userdata]]" end
+-- if we really need to return / serialize functions we could do it
+-- like cbreak@irc.freenode.net did here: http://lua-users.org/wiki/TablePersistence
+hook_wrapper_dump._function      = function(f) return "nil --[[function]]" end
+hook_wrapper_dump._nil           = function(n) return "nil" end
+hook_wrapper_dump._thread        = function(t) return "nil --[[thread]]" end
+hook_wrapper_dump._lightuserdata = function(l) return "nil --[[lightuserdata]]" end
+
+hook_wrapper_dump._table = function(t)
+    local buf = ''
+    if (hook_wrapper_dump.depth > 0) then
+        buf = buf .. '{\n'
+    end
+    hook_wrapper_dump.depth = hook_wrapper_dump.depth + 1;
+    for k,v in pairs(t) do
+        buf = buf..string.format('%s[%s] = %s;\n',
+              string.rep("\t", hook_wrapper_dump.depth - 1),
+              hook_wrapper_dump["_" .. type(k)](k),
+              hook_wrapper_dump["_" .. type(v)](v))
+    end
+    hook_wrapper_dump.depth = hook_wrapper_dump.depth - 1;
+    if (hook_wrapper_dump.depth > 0) then
+        buf = buf .. string.rep("\t", hook_wrapper_dump.depth - 1) .. '}'
+    end
+    return buf
+end
+
+function hook_wrapper(func_name, ...)
+    -- we have to ensure that nil arguments are restored properly for the
+    -- function call, see http://lua-users.org/wiki/StoringNilsInTables
+    local args = { n=select('#', ...), ... }
+    for i=1,args.n do
+        local val = assert(loadstring("return " .. args[i]),
+                         "argument "..args[i].." could not be evaluated")()
+        assert(val ~= nil or args[i] == "nil",
+               "argument "..args[i].." was evaluated to nil")
+        args[i] = val
+    end
+    local res = { _G[func_name](unpack(args, 1, args.n)) }
+    return hook_wrapper_dump._table(res)
+end
+
+
 function get_remote_unix_socket_command(host)
     return "socat"
 end
@@ -1170,19 +1276,20 @@ do
    local hook_functions = {}
    local supported_items = {
       "startup",
-      "start", "revision_received", "cert_received", "pubkey_received", "end"
+      "start", "revision_received", "revision_sent", "cert_received", "cert_sent",
+      "pubkey_received", "pubkey_sent", "end"
    }
 
    function _hook_functions_helper(f,...)
       local s = "continue"
       local v = nil
       for _,n in pairs(hook_functions) do
-	 if n[f] then
-	    s,v = n[f](...)
-	 end
-	 if s ~= "continue" then
-	    break
-	 end
+         if n[f] then
+            s,v = n[f](...)
+         end
+         if s ~= "continue" then
+            break
+         end
       end
       return v
    end
@@ -1195,50 +1302,59 @@ do
    function note_netsync_revision_received(...)
       return _hook_functions_helper("revision_received",...)
    end
+   function note_netsync_revision_sent(...)
+      return _hook_functions_helper("revision_sent",...)
+   end
    function note_netsync_cert_received(...)
       return _hook_functions_helper("cert_received",...)
    end
+   function note_netsync_cert_sent(...)
+      return _hook_functions_helper("cert_sent",...)
+   end
    function note_netsync_pubkey_received(...)
       return _hook_functions_helper("pubkey_received",...)
    end
+   function note_netsync_pubkey_sent(...)
+      return _hook_functions_helper("pubkey_sent",...)
+   end
    function note_netsync_end(...)
       return _hook_functions_helper("end",...)
    end
 
    function add_hook_functions(functions, precedence)
       if type(functions) ~= "table" or type(precedence) ~= "number" then
-	 return false, "Invalid type"
+         return false, "Invalid type"
       end
       if hook_functions[precedence] then
-	 return false, "Precedence already taken"
+         return false, "Precedence already taken"
       end
 
       local unknown_items = ""
       local warning = nil
       local is_member =
-	 function (s,t)
-	    for k,v in pairs(t) do if s == v then return true end end
-	    return false
-	 end
+         function (s,t)
+            for k,v in pairs(t) do if s == v then return true end end
+            return false
+         end
 
       for n,f in pairs(functions) do
-	 if type(n) == "string" then
-	    if not is_member(n, supported_items) then
-	       if unknown_items ~= "" then
-		  unknown_items = unknown_items .. ","
-	       end
-	       unknown_items = unknown_items .. n
-	    end
-	    if type(f) ~= "function" then
-	       return false, "Value for functions item "..n.." isn't a function"
-	    end
-	 else
-	    warning = "Non-string item keys found in functions table"
-	 end
+         if type(n) == "string" then
+            if not is_member(n, supported_items) then
+               if unknown_items ~= "" then
+                  unknown_items = unknown_items .. ","
+               end
+               unknown_items = unknown_items .. n
+            end
+            if type(f) ~= "function" then
+               return false, "Value for functions item "..n.." isn't a function"
+            end
+         else
+            warning = "Non-string item keys found in functions table"
+         end
       end
 
       if warning == nil and unknown_items ~= "" then
-	 warning = "Unknown item(s) " .. unknown_items .. " in functions table"
+         warning = "Unknown item(s) " .. unknown_items .. " in functions table"
       end
 
       hook_functions[precedence] = functions
