@@ -1,3 +1,12 @@
+-- Copyright (C) 2006 Timothy Brownawell <tbrownaw@gmail.com>
+--
+-- This program is made available under the GNU GPL version 2.0 or
+-- greater. See the accompanying file COPYING for details.
+--
+-- This program is distributed WITHOUT ANY WARRANTY; without even the
+-- implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
+-- PURPOSE.
+
 -- misc global values
 -- where the main testsuite file is
 srcdir = get_source_dir()
@@ -439,6 +448,10 @@ function samefile(left, right)
  end
 end
 
+function samefilestd(left, right)
+   return samefile(testdir .. "/" .. test.name .. "/" .. left, right)
+end
+
 function samelines(f, t)
   local fl = {}
   for l in io.lines(f) do table.insert(fl, l) end
@@ -602,6 +615,7 @@ function post_cmd(result, ret, stdout, stderr, ident)
   log_file_contents(ident .. "stdout")
   L("stderr:\n")
   log_file_contents(ident .. "stderr")
+  L("exit code: " .. result .. "\n")
   if result ~= ret and ret ~= false then
     err("Check failed (return value): wanted " .. ret .. " got " .. result, 3)
   end
@@ -854,8 +868,13 @@ function run_tests(debugging, list_only, run_dir, logname, args, progress)
      set_env(name,"C")
   end
 
-  -- no test suite should touch the user's ssh agent
+  -- no test suite should touch the user's ssh agent or display
   unset_env("SSH_AUTH_SOCK")
+  unset_env("DISPLAY")
+
+  -- tests do not use (interactive) editors for commits
+  unset_env("EDITOR")
+  unset_env("VISUAL")
 
   logfile = io.open(logname, "w")
   chdir(run_dir);
@@ -1061,6 +1080,16 @@ function run_one_test(tname)
    test.bglist = {}
    test.log = io.open("tester.log", "w")
 
+   -- Sanitize $HOME.  This is done here so that each test gets its
+   -- very own empty directory (in case some test writes stuff inside).
+   unlogged_mkdir("emptyhomedir")
+   test.home = test.root .. "/emptyhomedir"
+   if ostype == "Windows" then
+      set_env("APPDATA", test.home)
+   else
+      set_env("HOME", test.home)
+   end
+
    L("Test ", test.name, "\n")
 
    local driverfile = testdir .. "/" .. test.name .. "/__driver__.lua"
