@@ -26,6 +26,7 @@ addfile("file_0", "original: root file_0")
 addfile("dir_a/file_a", "original: dir_a file_a")
 addfile("dir_b/file_b", "original: dir_b file_b")
 commit()
+rev1 = base_revision()
 
 -- Test that 'automate inventory' shows all directories
 check(mtn("automate", "inventory"), 0, true, false)
@@ -37,6 +38,7 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
    status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -44,6 +46,7 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
    status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -51,6 +54,7 @@ index = check_inventory (parsed, index,
  old_type = "file",
  new_type = "file",
   fs_type = "file",
+    birth = rev1,
    status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -58,6 +62,7 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
  status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -65,13 +70,20 @@ index = check_inventory (parsed, index,
  old_type = "file",
  new_type = "file",
   fs_type = "file",
+    birth = rev1,
    status = {"known"}})
 
+index = check_inventory (parsed, index,
+{   path = "emptyhomedir",
+ fs_type = "directory",
+  status = "unknown"})
+
 index = check_inventory (parsed, index,
 {    path = "file_0",
  old_type = "file",
  new_type = "file",
   fs_type = "file",
+    birth = rev1,
    status = {"known"}})
 
 -- skip the test files in root
@@ -89,6 +101,7 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
    status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -96,6 +109,7 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
    status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -103,13 +117,20 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
    status = {"known"}})
 
+index = check_inventory (parsed, index,
+{   path = "emptyhomedir",
+ fs_type = "directory",
+  status = "unknown"})
+
 index = check_inventory (parsed, index,
 {    path = "file_0",
  old_type = "file",
  new_type = "file",
   fs_type = "file",
+    birth = rev1,
    status = {"known"}})
 
 -- skip tester-generated files
@@ -130,6 +151,7 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
    status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -137,6 +159,7 @@ index = check_inventory (parsed, index,
  old_type = "file",
  new_type = "file",
   fs_type = "file",
+    birth = rev1,
    status = {"known"}})
 
 -- prove that we checked all the output
@@ -155,6 +178,7 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
    status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -176,6 +200,7 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
  status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -183,6 +208,7 @@ index = check_inventory (parsed, index,
  new_type = "file",
  old_path = "dir_a/file_a",
   fs_type = "none",
+    birth = rev1,
    status = {"rename_target", "missing"}})
 -- "missing" because of --bookkeep-only
 
@@ -191,6 +217,7 @@ index = check_inventory (parsed, index,
  old_type = "file",
  new_type = "file",
   fs_type = "file",
+    birth = rev1,
    status = {"known"}})
 
 checkexp ("checked all", #parsed, index-1)
@@ -208,6 +235,7 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
    status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -228,6 +256,7 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
  status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -235,6 +264,7 @@ index = check_inventory (parsed, index,
  new_type = "file",
  old_path = "dir_a/file_a",
   fs_type = "file",
+    birth = rev1,
    status = {"rename_target", "known"}})
 
 index = check_inventory (parsed, index,
@@ -242,6 +272,7 @@ index = check_inventory (parsed, index,
  old_type = "file",
  new_type = "file",
   fs_type = "file",
+    birth = rev1,
    status = {"known"}})
 
 checkexp ("checked all", #parsed, index-1)
@@ -261,6 +292,7 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
    status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -268,6 +300,7 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
    status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -275,8 +308,14 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
    status = {"known"}})
 
+index = check_inventory (parsed, index,
+{   path = "emptyhomedir",
+ fs_type = "directory",
+  status = "unknown"})
+
 index = check_inventory (parsed, index,
 {    path = "file_0",
  old_type = "file",
@@ -305,6 +344,7 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
    status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -312,6 +352,7 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
    status = {"known"}})
 
 index = check_inventory (parsed, index,
@@ -319,8 +360,14 @@ index = check_inventory (parsed, index,
  old_type = "directory",
  new_type = "directory",
   fs_type = "directory",
+    birth = rev1,
    status = {"known"}})
 
+index = check_inventory (parsed, index,
+{   path = "emptyhomedir",
+ fs_type = "directory",
+  status = "unknown"})
+
 index = check_inventory (parsed, index,
 {    path = "file_0",
  old_type = "file",
@@ -376,6 +423,7 @@ index = check_inventory (parsed, index,
  new_type = "directory",
  old_path = "dir_a",
   fs_type = "directory",
+    birth = rev1,
    status = {"rename_target", "known" }})
 
 index = check_inventory (parsed, index,
@@ -383,6 +431,7 @@ index = check_inventory (parsed, index,
  new_type = "file",
  old_path = "dir_a/file_a",
   fs_type = "file",
+    birth = rev1,
    status = {"rename_target", "known" }})
 
 checkexp ("checked all", #parsed, index-1)
@@ -424,6 +473,7 @@ index = check_inventory (parsed, index,
  new_type = "directory",
  old_path = "dir_a",
   fs_type = "directory",
+    birth = rev1,
    status = {"rename_target", "known" }})
 
 checkexp ("checked all", #parsed, index-1)
