@@ -18,15 +18,15 @@ testURI="file:" .. test.root .. "/test.db"
 -- all of these inherit options settings from the current _MTN dir
 -- unless they override them on the command line
 
-check(nodb_mtn("--branch=testbranch", "clone", testURI, "test_dir1"), 0, false, false)
-check(nodb_mtn("--branch=testbranch", "clone", testURI, "--revision", rev, "test_dir2"), 0, false, false)
-check(nodb_mtn("--db=" .. test.root .. "/test-old.db", "--branch=testbranch", "clone", testURI, "test_dir3"), 0, false, false)
-check(nodb_mtn("--db=" .. test.root .. "/test-old.db", "--branch=testbranch", "clone", testURI, "--revision", rev, "test_dir4"), 0, false, false)
+check(nodb_mtn("clone", testURI, "testbranch", "test_dir1"), 0, false, false)
+check(nodb_mtn("clone", "--revision", rev, testURI, "testbranch", "test_dir2"), 0, false, false)
+check(nodb_mtn("--db=" .. test.root .. "/test-old.db", "clone", testURI, "testbranch", "test_dir3"), 0, false, false)
+check(nodb_mtn("--db=" .. test.root .. "/test-old.db", "clone", testURI, "testbranch", "--revision", rev, "test_dir4"), 0, false, false)
 
 -- checkout fails if the specified revision is not a member of the specified branch
-check(nodb_mtn("--branch=foobar", "clone", testURI, "--revision", rev, "test_dir5"), 1, false, false)
+check(nodb_mtn("clone", testURI, "--revision", rev, "foobar", "test_dir5"), 1, false, false)
 check(mtn("cert", rev, "branch", "foobar"), 0, false, false)
-check(nodb_mtn("--branch=foobar", "clone", testURI, "--revision", rev, "test_dir6"), 0, false, false)
+check(nodb_mtn("clone", testURI, "--revision", rev, "foobar", "test_dir6"), 0, false, false)
 
 
 for i = 1,2 do
